/*
Java Hummer Project
File: DefaultAliasInfo.java
Date: 4-21-99
Author: Chad Ward
Description:  Dataobject holding all the information about a given
              alias in a DefaultKeystore.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.dataobject;


import java.util.Hashtable;
import java.security.Key;

/**
AliasInfo object for DefaultKeyStore.
@author Chad Ward
*/
public final class DefaultAliasInfo extends AliasInfo {
	/**
	Default constructor. Copies password into local value using System.arraycopy
	@param certificate_honored A value that is only meaningful for _SERVER_ALIAS_TYPE
		accounts.  It is ignored for other account types.
	@see AliasInfo#_certificate_honored
	*/
  public DefaultAliasInfo(int alias_type, boolean certificate_honored,
                          byte[] password, Hashtable access_table) {
    super(alias_type, access_table);
    _new_user_validator = new NewUserValidator(password);
    if(alias_type == AliasInfo._SERVER_ALIAS_TYPE) {
      this._certificate_honored = certificate_honored;
    }
    _keys = new Key[1];
  }

	/**
	Sets the PublicKey for the account, sets the backup timestamp to -1,
	and sets the NewUserValidator object to null after calling clear.
	*/
  public boolean initializeAccount(Object public_key) {
    this._keys[_PUBLIC_KEY_TYPE] = (Key)public_key;
    _backup_timetamp = -1;
    _new_user_validator.clear();
    _new_user_validator = null;
    return true;
  }
}