/*
Java Hummer Project
File: Level.java
Date: 11-23-98
Author: Jesse C. McConnell
Description:  This file contains the level object. This object contains the
             needed information for a levels filter.

Adapted from Level.h from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.dataobject;

import java.net.InetAddress;

/**
This class contains the needed information for a levels filter. When created
messages can be filtered out on the criteria of whether they exist within the
specified InetAddress range, and match to Trust, Cooperation and Integrity
levels.

@author Jesse C. McConnell
*/
public class Level extends Object {
  /**
  The starting InetAddress in a range, or the unique InetAddress is the boolean
  value explicit is set to true.

  Assumption: The start_host_ip is the low end of the range.
  */
  public InetAddress start_host_ip;
  /**
  The ending InetAddress of a range.

  Assumption: The end_host_ip is this high end of the range, or explicit is set
              to false.
  */
  public InetAddress end_host_ip;
  /**
  Boolean value representing true if the start_host_ip InetAddress is unique or
  false if there is an end_host_ip.
  */
  public boolean explicit;
  /**
  Character representing the Trust Level to filter the Message on.  This should be
  standardized with the other trust values in Hummer to int or Integer.
  */
  public char trust_level;
  /**
  Character representing the Cooperation Level to filter the Message on.  This
  should be standardized with the other cooperation values in Hummer to int or
  Integer.
  */
  public char coop_level;
  /**
  Character representing the Integrity Level to filter the Message on.  This
  should be standardized with the other interity values in Hummer to int or
  Integer.
  */
  public char integ_level;
}
