
/*
Java Hummer Project
File: Peer.java
Date: 1-16-99
Author: Jesse C. McConnell
Description: This is a basic object for Peer communication.

 Copyright (C) 1997-1999 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


package edu.uidaho.hummer.dataobject;

import java.io.Serializable;
import java.util.Vector;
import java.util.Date;
import edu.uidaho.hummer.dataobject.Message;

/**
This is the basic Peer Object.  When peer communication takes place,
it is a serialization of this object.  All communication between 
Servers and Clients that deal with passing Peer communication, are 
passing these.

@author Jesse C. McConnell

@serial
*/

public class Peer implements Serializable {


  /**
  This is the action the message is representing.
  <p>
  Local -> Server<br>
  subscribe - Local subscribing to the Peer Group on Server<br>
  unsubscribe - Local unsubscribing from the Peer Group on Server<br>
  message - Local is sending a Message Object<br>
  <br>
  Server -> Local<br>
  statements - Server sending SQL Statements for setting up the Local 
  database<br>
  unsubscribed - Server telling Local they are unsubscribed<br>
  error - Some sort of error occured.  To be fleshed out later.<br>

  @serial
  */
  public String action = new String(); 

  /**
  Vector of database java.sql.Statement.

  @serial
  */
  public Vector statements = new Vector();
  
  /**
  String representing the Peer group.

  @serial
  */
  public String peer_group = new String();

  /**
  Message object for the passing of a message into the Peer Server System.

  @serial
  */
  public Message message = new Message();
  
  /**
  Date object representing the construction time of the Peer object.   

  @serial
  */
  public Date date = new Date();

}
