/****************************************************************
Interface.java
CS 481 - Team A
Written: Spring 1997
Revised: Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Revised by: Jesse C. McConnell
Description: This is a cheesy interface thrown together to display 
             hummer message forwarded to the alert interface. 
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.gui.hac;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Properties;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;
import java.net.*;
import java.io.*;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.server.InterfaceServer;
import edu.uidaho.hummer.util.StaticMethods;

/**
This is a cheesy interface thrown together to display 
hummer message forwarded to the alert interface. 
@author Pete Neisen, Jesse McConnell
*/
public class Interface extends Frame {

  private List _lst;
  private ButtonPanel _pnl;
  private Vector _message_list;
  private InetAddress _addr;

  protected Properties _constants;

  protected Button _viewbutton, _removebutton, _clearbutton, _quitbutton;


  public static void main(String argv[]) {
    Interface interface_object = new Interface();
  }

  public Interface() {
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("Interface:main:Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("Interface:main:File Hummer.properties not valid.");
      System.exit(0);
    }

    _message_list = new Vector(10,5);
    
    try {
      _addr= InetAddress.getLocalHost();
    }
    catch (IOException ioe) { 
      System.err.println("Interface:main:" + ioe);
      System.exit(0);
    }

    _lst = new List(10, false);
    _pnl = new ButtonPanel(_lst, _message_list);

    _pnl.setLayout(new FlowLayout());

    _pnl.add(_viewbutton = new Button("View Message"));
    _viewbutton.setActionCommand("1");
    _viewbutton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        int index;
        Message tmp_msg;
        index = _lst.getSelectedIndex();

        if(index >= 0){
          tmp_msg = (Message)_message_list.elementAt(index);
          MessageFrame view_window = new MessageFrame(tmp_msg);	
        }
      }
    });

    _pnl.add(_removebutton = new Button("Remove Message"));
    _removebutton.setActionCommand("2");
    _removebutton.addActionListener(_pnl);

    _pnl.add(_clearbutton = new Button("Clear Messages"));
    _clearbutton.setActionCommand("3");
    _clearbutton.addActionListener(_pnl);

    _pnl.add(_quitbutton = new Button("Quit"));
    _quitbutton.setActionCommand("4");
    _quitbutton.addActionListener(_pnl);

    this.setSize(400,300);
    _pnl.validate();
    this.setTitle("Hummer Alert Interface - " + _addr.getHostName());
    this.setLayout(new BorderLayout());
    this.add("Center", _lst);
    this.add("South", _pnl);
    
    this.setVisible(true);

    try {
      Integer value = new Integer(_constants.getProperty(
		  		  "port.ALERT_INTERFACE_PORT"));   
      new InterfaceServer(value.intValue(), this); 
    }
    catch (NumberFormatException nfe) {
      System.err.println("Interface:main:Bad ALERT_INTERFACE_PORT" + 
			 " in Hummer.properties");
      System.exit(0);
    }
  }

  public List getList() { return _lst; }
  public ButtonPanel getButtonPanel() { return _pnl; }
  public Vector getMessageList() { return _message_list; }

  /**
     Tokenize the message into a message object and add it to the list.
   */
  public void addToList(Message new_message) {
    StringTokenizer strtok;

    _message_list.insertElementAt(new_message,0);
    _lst.add(new_message.description,0);
    _lst.select(0);

  }
}


class ButtonPanel extends Panel implements ActionListener
{

  private List _lst;
  private Vector _message_list;

  public ButtonPanel(List lst, Vector message_list) {
    this._lst = lst;
    this._message_list = message_list;
  }

  public void actionPerformed(ActionEvent evt)
  {
    Integer event_number = new Integer(evt.getActionCommand());
    int index;

    switch(event_number.intValue()) {

    case 2:
      index = _lst.getSelectedIndex();

      if(index >= 0){
        _lst.remove(index);
        _message_list.removeElementAt(index);
        _message_list.trimToSize();
        _lst.select(0);
      }
      break;

    case 3:
      int end_index;
      end_index = _lst.getItemCount();
      
      
      if (end_index > 0) {
        _lst.removeAll();
        _message_list.removeAllElements();
      }
      break;
    case 4:
      System.exit(0); 
    }
  }
}



class MessageFrame extends Frame implements ActionListener {

  private Button _close;

  public MessageFrame(Message new_msg) {
    
    super(new_msg.description);

    String str;
    TextArea txtarea;
    SimpleDateFormat date_format = new SimpleDateFormat("MM-dd-yyyy");
    SimpleDateFormat time_format = new SimpleDateFormat("hh:mm:ss");

    setSize(450,200);
    _close = new Button("Close");
    _close.setActionCommand("1");
    _close.addActionListener(this);

    add("South",_close);

    str = "Description: "+new_msg.description+
      "\nTrust Level: "+new_msg.trust+
      "\nCooperation Level: "+new_msg.cooperation+
      "\nIntegrity Level: "+new_msg.integrity+
      "\nDate: " + date_format.format(new_msg.date_time) +
      "\nTime: " + time_format.format(new_msg.date_time) +
      "\nOriginating Host: " + new_msg.host_IP +
      "\nFilename: " + new_msg.filename +
      "\nTool: " + new_msg.tool + "\n";
    txtarea = new TextArea(str,325,175);
    txtarea.setBackground(Color.gray);
    add("Center", txtarea); 
    
    this.setVisible(true);
  }

  public void actionPerformed(ActionEvent evt) {
    
    if((evt.getActionCommand()).equals("1")) {
      this.setVisible(false);
      this.dispose();
    }
  }
}