/*
Java Hummer Project
File: HuiAuthentication.java
Date: 5-6-99
Author: Andy Hoobing and Joel Marlow
Description:  This file has all the code for the Hummer User Interface.  I used
NetBeans (build 112) to make most of this code.  The rest is done by hand.
This is a rather large file.

Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 *
 * @author Andy Hoobing, Joel Marlow, Chad Ward
 * @version Alpha
 */

package edu.uidaho.hummer.gui.hui;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.socket.HummerClientSocket;




public class HuiAuthentication {

  /** This holds the current key store */
public HummerKeyStore _key_store;

  /** Has the keystore been loaded properly? */
public boolean _key_store_loaded = false;

  /** The personal config object for the current user */
public PersonalConfig _personal;

  /** Is there a connection with config server? */
public boolean _made_connection;

  /** The current address of the server */
InetAddress _sub_addr;

  /** The client socket object for the current connection*/
public Socket _client_socket;

  /** The current config object for the current session*/
Config _config;

  /** The username of the current user */
public String _username = null;

  /** The password of the current user */
public String _password = null;


public String _address;
public String _peer_address = null;

  //++Needed for authentication
HummerClientSocket _dcs = null;




  /**

  This constructor will look for the server on the local host and try
and establish a connection with the server.  It will load the key store
and create a new hummer client socket 
@param username The username of the current user
@param password The password of the current user
  */


public HuiAuthentication(String username, String password) {

 _username = username;
 _password = password;

 // System.out.println("PASSWORD: " + password);
 String address = new String();
    //++This try and catch is needed for authentication
    try {
      _address = InetAddress.getLocalHost().getHostAddress();
      //System.out.println("New Authenticator");
      _key_store = HummerKeyStore.getInstance();
      _key_store.load(false, _username);
      _key_store_loaded = true;
    }

    catch (FileNotFoundException e) {
      //System.err.println("HuiAuthentication:init: " + e);
      javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: FileNotFoundException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      _key_store_loaded = false;
    }
    catch(KeyStoreException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication: Error while loading HummerKeyStore.", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }
    catch(ClassNotFoundException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: ClassNotFoundException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }
    catch(StreamCorruptedException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: StreamCorruptedException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }
    catch(UnknownHostException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: UnknownHostException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }
    catch(OptionalDataException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: OptionalDataException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }
    catch(IOException e) {
      //System.err.println("HuiAuthentication:init: Error while loading HummerKeyStore: " + e);
javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: IOExcetpion", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      return;
    }

    if(!_key_store_loaded) {
      try {
	// javax.swing.JOptionPane.showInternalMessageDialog(, "HummerKeyStore not found.  Loading new key store...");
	//        System.out.println("Loading");
        //System.out.println("Password: " + password);
        _key_store.load(password.getBytes());
        _key_store.store(false, _username);
        //System.out.println("Loaded");
        _key_store_loaded = true;
         
      }
      catch(KeyStoreException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
        javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: KeyStoreException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(ClassNotFoundException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: ClassNotFoundException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(CertificateException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: CertificateException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(NoSuchAlgorithmException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
      javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: NoSuchAlgorithmException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(java.security.NoSuchProviderException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: NoSuchProviderException ", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(StreamCorruptedException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: StreamCorruptedException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(UnknownHostException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
     javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: UnknownHostException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(OptionalDataException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: OptionalDataException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        return;
      }
      catch(IOException e) {
        //System.err.println("HuiAuthentication:init: Error while creating HummerKeyStore: " + e);
     javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: IOException .", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      }

      catch(Exception e) {
  javax.swing.JOptionPane.showMessageDialog(null, "HuiAuthentication Constructor: probably caught null pointer" , "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      }
    }

    //  catch(StreamCorruptedException e){}
  //++Needed for authentication
   
    if(password == null) {
      //System.out.println("Setting to NULL");
      _personal = new PersonalConfig(username, null);
    }
    else{
      //System.out.println("Not Setting to NULL");
      _personal = new PersonalConfig(username, password.getBytes());
    //    _personal = new PersonalConfig("admin", null);
    }

} // end constructor








  /** This function will open a connection with the Config server 
     @param sub_name The IP address of the server to connect to.
     @param port The port number the server is on
*/

  public boolean openServer(InetAddress sub_name, int _port) {
    // Create new socket 
    if (!_made_connection) {
      try  {
        _client_socket = new Socket(sub_name, _port);
        _peer_address = sub_name.getHostAddress();;
        //System.out.println("Connected to chat server on " + sub_name);
        _made_connection = true;
      }
      catch(UnknownHostException e) {
        //System.err.println("HuiAuthentication:openServer:Host not found:  " + e.getMessage());
        javax.swing.JOptionPane.showMessageDialog(null, "OpenServer: Host not Found", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        _made_connection = false;
      }
      catch(IOException e) {
        javax.swing.JOptionPane.showMessageDialog(null, "OpenServer: IOException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
        _made_connection = false;
      }
    }
    return true;
  } //openServer



  /**
  Adds a new manager given a password.
  */
  public void addManager(String manager_address, byte[] password) {
//    byte[] byte_password = new byte[password.length];
//    for(int i = 0; i < password.length; i++) {
//      byte_password[i] = (byte)password[i];
//    }
    //System.out.println("HuiAthentication:password: " + (new String(password)));
    PersonalConfig manager_config = new PersonalConfig(_dcs.getCurrentPartnerAlias(), password);
    try {
      _dcs.addManager(manager_config, manager_address);
    }
    catch(IOException e) {
      javax.swing.JOptionPane.showMessageDialog(null, "AddManager: IOException", "FYI",  javax.swing.JOptionPane.ERROR_MESSAGE);
    }
    catch(edu.uidaho.hummer.util.socket.HummerSocketException e) {
      javax.swing.JOptionPane.showMessageDialog(null, "AddManager: HummerSocketException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
    }
  }


  /**
  This method validates the connection for the user

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  public boolean validateConnection(String certificate_location){

  //++This try andatch is needed for authentication

    //System.out.println("KeyStoreLoaded: " + _key_store_loaded);
    try {
      //System.out.println("Personal Config is new: " + _personal.isNewAccount());
      _dcs = new HummerClientSocket(_personal, _client_socket, _key_store);
      //      System.out.println("before validation");

      
//String user_input = null;
//    BufferedReader input_stream = new BufferedReader(new InputStreamReader(System.in));
//    while(user_input == null || (!user_input.equals("y") && !user_input.equals("n"))) {
//      System.out.println("is new account (y/n):");
//      user_input = input_stream.readLine().toLowerCase();
//    }

//    if(user_input.equals("n")) {
//      _personal.initializeAccount();
      
      //    _dcs.validation("129.101.54.53"); this should be the same as below
      //System.out.println("CERT: " + certificate_location);

      _dcs.validation(certificate_location);

      
      _key_store.store(false, _username);
    }

    catch (Exception e) {
      System.err.println("HuiAuthentication:validateConnection: " + e);
     javax.swing.JOptionPane.showMessageDialog(null, "ValidateConnection: Connection Not Validated", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
     //e.printStackTrace();
      //      System.out.println("I have an error here!");
    }

  try {
        _key_store.store(false, _username);
      }
      catch (Exception e) {           
	// System.err.println("HuiAuthentication:validateConnection: " + e);
 javax.swing.JOptionPane.showMessageDialog(null, "ValidateConnection: Problems Writing KeyStore", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      }     
      return true;
  } // validateConnection




  /**
     This method is the one used in order to send and get data from the server
     @param config The config object with the action messages in it.
     @returns Config The data sent back from the database.
     */
  public Config sendData(Config config) {

  //++Needed for authentication
    Response response = new Response();

  //++Needed for authentication
    response._type = Response._OK;
  //++Needed for authentication
    response._message = config;
    // End of protocol mesage, and close out connection
  //++This try and catch is needed for authentication
    if(_made_connection) {
    try  {
      _dcs.writeResponseObject(response);
      response = _dcs.readResponseObject();
      _config = (Config)response._message;
    }
    catch(UnknownHostException e) {
      //System.err.println("HuiAuthentication:sendData:Host not found:  " + e.getMessage());
 javax.swing.JOptionPane.showMessageDialog(null, "SendData: No Hummer Found", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
       
        System.exit(1);
    }
    catch(IOException e) {
      //System.err.println("HuiAuthentication:sendData: " + e.getMessage());
 javax.swing.JOptionPane.showMessageDialog(null, "SendData: IOException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    catch (Exception e) {
      //System.err.println("HuiAuthentication:sendData: " + e);
        javax.swing.JOptionPane.showMessageDialog(null, "SendData: Exception", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      //e.printStackTrace();
    }
    }// end if
    return _config;
  } //sendData








  /**
  This method sends closes the _client_socket to the server.
  */
  public void closeServer() {
    Config config = new Config();
    //++Needed for authentication
    Response response = new Response();

    config.data.addElement("close");
    //++Needed for authentication
    response._type = Response._OK;
    //++Needed for authentication
    response._message = config;

    if(_made_connection) {
    try {
      _dcs.writeResponseObject(response);
      _dcs.close();
      //System.out.println("Im closing the server now");
      _made_connection = false;
    }
    catch(IOException e) {
      //      System.err.println("HuiAuthentication:closeServer: " + e.getMessage());
      javax.swing.JOptionPane.showMessageDialog(null, "CloseServer: IOException", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    catch (Exception e) {
      //System.err.println("HuiAuthentication:sendData: " + e);
       javax.swing.JOptionPane.showMessageDialog(null, "CloseServer: Exception", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      //e.printStackTrace();
    } 
    } // end if
  }//closeServer




  /** This function will get the current access table for the current user.
     @param username Name of the user to get the table for (admin only)
     @parm alias Alias of the current user
     @param signer_alias used to get the access table when you are not admin
  */

  public Hashtable getAccessTable(String username, String alias, String signer_alias) {

    Hashtable temp_hash = new Hashtable();
    //System.out.println("ALIAS: " + alias);    
    //System.out.println("USERNAME: " + username);

    if(_made_connection) {
      try{
	if(alias.equals("admin")) {
	  //System.out.println("getting admin data");
	  temp_hash = _dcs.getAccessTable(username);
	  //System.out.println("HASHTB: " + temp_hash.toString());
	}
            else {
	      //  System.out.println("before getting certificate");
             if(signer_alias.equals("Local")) {
                InetAddress tempaddr = _client_socket.getInetAddress();         
                signer_alias = tempaddr.getHostAddress();
	 
	HummerCertificate tempcert = _key_store.getHummerCertificate(signer_alias);
	      // end if
	//System.out.println("After getting certificate");
               temp_hash = tempcert.getAccessTable();
               //System.out.println("Hashtable: " + temp_hash.toString());
	      }
	       //System.out.println("After getting access table");
	    } // end else
 
      }

      catch(UnloadedKeyStoreException e) {
  javax.swing.JOptionPane.showMessageDialog(null, "getAccessTable: UnloadedKeyStoreException .", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      }

      catch(UnknownAliasException e) {
  javax.swing.JOptionPane.showMessageDialog(null, "getAccessTable: UnknownAliasException .", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
      }
      catch(Exception e){
       javax.swing.JOptionPane.showMessageDialog(null, "getAccessTable: Problem Getting Access Table", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
}

    }

    return temp_hash;

  }


  public void setAccessTable(String username, Hashtable table) {

    if(_made_connection){
      try{
	_dcs.setAccessTable(username, table);
      }

      catch(Exception e){
       javax.swing.JOptionPane.showMessageDialog(null, "setAccessTable: Problem Setting Access Table Values", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
}


    } // end if

  }




  /** This will remove an alias from the server 
        @param username name of user to remove
*/

  public void removeAlias(String username) {

    if(_made_connection)
      try{
	_dcs.deleteAlias(username);
      }

    catch(Exception e){
     javax.swing.JOptionPane.showMessageDialog(null, "removeAlias: Problem removing alias", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
}

  }





  /** This function will get the current alias list of the server 
       @returns Vector vector of aliases
*/


  public Vector getAliasList() {

    Vector temp_vector = new Vector();

    if(_made_connection){
      try{
      temp_vector = _dcs.getAliasList();
      }

      catch(Exception e){
       javax.swing.JOptionPane.showMessageDialog(null, "getAliasList: problem getting alias list", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
}

      }

    return temp_vector;

  } // end getAliasList






  /** This function will get the curreent certificate alias list */

  public Vector getCertificateAliasList() {

    Vector return_data_vector = new Vector();

    if(_key_store_loaded) {
      try{
      return_data_vector = _key_store.getCertificateAliasList();
      }
      catch(Exception e){
       javax.swing.JOptionPane.showMessageDialog(null, "getCertificateAliasList: problem getting certificate alias list", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
}
    }

    return return_data_vector;

  }






  /** This will add an alias to the server */
  public void addUserAlias(String username, byte[] password) {

    if(_made_connection) {
      try{
	//System.out.println("Adding Alias");
	_dcs.addUserAlias(username, password);

      } // end try
      catch(Exception e){
        javax.swing.JOptionPane.showMessageDialog(null, "addUserAlias: problem adding user alias", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
     }

      } // end if

    }  // end addUserAlias






  /**
     This method will add the alias of the peer to the moderator's key store
   */
  public void addPeerAlias(String alias, String password){
    try{
      _dcs.addPeerAlias(alias, password.getBytes());
    }
    catch (Exception e) {
      //System.err.println("HuiAuthentication:addPeerAlias: " + e);
      //e.printStackTrace();
       javax.swing.JOptionPane.showMessageDialog(null, "addPeerAlias: problem adding peer alias", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
    } 
  }



  /**
     This method will change the certificates between the moderator
     and the one subscribing to the peer group
   */
  public void addPeerModerator(String moderator, String password) {
    PersonalConfig personal;
    try{
      personal = new PersonalConfig(_peer_address, password.getBytes());
      _dcs.addPeerModerator(personal, moderator);
    }
    catch (Exception e) {
      // System.err.println("HuiAuthentication:addPeerModerator: " + e);
      //e.printStackTrace();
     javax.swing.JOptionPane.showMessageDialog(null, "addPeerModerator: problem adding moderator", "FYI", javax.swing.JOptionPane.ERROR_MESSAGE);
    } 

  }
}
