/****************************************************************
Server.java
CS 481 - Team A
Written: Spring 1997
Revised: String 1998
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Revised by: Jesse C. McConnell, Chad Ward
Description: This is a cheesy interface thrown together to display 
             hummer message forwarded to the alert interface. 
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStoreException;
import edu.uidaho.hummer.gui.hac.Interface;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;


/**
   Interface is a little window which displays information being 
   accepted by the Hummer server.  Interface server is what talks
   to hummer, to find out what it going on.
   
   <h4>Class Usage and Flow</h4>
   <ul>
   <li> Create a server listening on a port.
   <li> Accept connections and bind to a new client when
        it shows up (very insecurely).
   </ul>
@author Pete Neisen, Jesse McConnell, Chad Ward
*/
public class InterfaceServer extends Thread {
  
	/**
	The socket which the server will use to listen on.
	*/
  private ServerSocket _listen_socket;

	/**
	The port the server will open a socket connection to listen on.
	*/
  private int _port;

	/**
	The Interface object that should be notified when a new message arrives.
	*/
  private Interface _interface;

	/**
	The key store used for validating communication with the MessagingServer
	*/
  private HummerKeyStore _key_store;

	/**
	The default constructor starts the server running on a port.

	<h4> Constructor Usage and Flow </h4>
	<ul>
	<li> Create a new socket on the port _port.
 	</ul>
	@param port this is the port the server will run on.
	*/
  public InterfaceServer(int port, Interface interface_object) {
    this._port = port;
    this._interface = interface_object;

    try {
      _key_store = HummerKeyStore.getInstance();
      _key_store.load(true, null);
    }
    catch(IOException e) {
      System.err.println("InterfaceServer:init: Could not create HummerKeyStore: " + e);
    }
    catch(KeyStoreException e) {
      System.err.println("InterfaceServer:init: Could not create HummerKeyStore: " + e);
    }
    catch(ClassNotFoundException e) {
      System.err.println("InterfaceServer:init: Could not create HummerKeyStore: " + e);
    }
   


    try {
      _listen_socket = new ServerSocket(_port);
    }
    catch (IOException E) {
      System.err.println("InterfaceServer:" + E);
      System.exit(1);
    }
    this.start();
  }

  /**
     Run the server.
  */
  public void run()
  {
    try{
      
      while(true) {
        Socket client_socket = _listen_socket.accept();
        InterfaceHandler c = new InterfaceHandler(client_socket, _key_store, _interface);
      }
    }
    catch(IOException E){
      System.err.println("InterfaceServer:" + E);
    }
  }
}