/*
Java Hummer Project
File: MessageFilterCon.java
Date: 11-23-98
Author: Jesse C. McConnell
Description: This is the class definition of the message filter container
             object. It stores and handles the message filter objects. It
             also handles the regualr expression matching of the filters
             to a message.

Adapted from MessageFilterCon.C from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.util.Vector;
import java.util.Enumeration;
import edu.uidaho.hummer.dataobject.*;

/**
This is the class definition of the message filter container object. It
stores and handles the message filter objects. It also handles the regualr
expression matching of the filters to a message.

@author Jesse C. McConnell
*/
public class MessageFilterCon extends Object {
  /**
  Vector which should contain only MessageFilter objects.
  */
  private Vector _msg_filters;


  /**
  Basically initializes the _msg_filters Vector.
  */
  public MessageFilterCon() {
    _msg_filters = new Vector();
  }

  /**
  Adds a message filter object to the container.
  */
  public void addFilter(MessageFilter newfilter) {
    _msg_filters.addElement(newfilter);
  }

  /**
  This function returns a Vector of MessageFilter's that match the message
  description.

  @param String representing the description of the Message object that all
  of the MessageFilter's are tested against.
  @return Vector of MessageFilters that match some part of the message String.
  */
  public Vector matchExp(String message) {
    Vector matched = new Vector();

    Enumeration enum = _msg_filters.elements();

    while(enum.hasMoreElements()) {
      MessageFilter test_filter = (MessageFilter)enum.nextElement();

      if (Match(message, test_filter.regexp)) {
        matched.addElement(test_filter);
      }
    }
    return matched;
  }

  /**
  Checks to see if the string pattern is in the to_match string.

  @param to_match String that is to be checked if it contains the pattern.
  @param pattern String that is the pattern to check the to_match String
  against.
  @return True if pattern is in to_match, false if not.
  */
  private boolean Match(String to_match, String pattern) {
    int test;

    test = to_match.indexOf(pattern);

    if (test == -1) {
      return false;
    }
    else {
      return true;
    }
  }
};
