/*
Java Hummer Project
File: Test.java
Date: 2-17-99
Author: Chad Ward
Description:  Parent class for all tests run.  Classes extending
              This class only have to define the testDefinition method,
              and then use the runTest method to perform the test.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.client;

import java.io.PrintStream;
import java.net.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import edu.uidaho.hummer.test.Test;
import edu.uidaho.hummer.client.ConfigI;
import edu.uidaho.hummer.dataobject.Config;
import edu.uidaho.hummer.util.StaticMethods;

public class ConfigITest extends Test implements Runnable {

  private static int _port;
  private static String _config_server_url, _sql_driver, _sql_user, _sql_password;
  private static Properties _constants = System.getProperties();
  private static InetAddress _local_host;
	/**
	List of table names.  The values for these tables are found in
	_db_values.
	@see ConfigITest#fillDB()
	*/
  private static final String[] _db_tables = { "accept_list",
                                               "filter",
                                               "forward_list",
                                               "kill_file",
                                               "levels",
                                               "subordinates"};

  private static final int _ACCEPT_LIST = 0, _FILTER = 1, _FORWARD_LIST = 2,
                           _KILL_FILE = 3, _LEVELS = 4;

	/**
	Correct values for accept_list test
	*/
  private static Vector[] _correct_values = new Vector[_db_tables.length];
  
	/**
	The values to be placed in the tables of the database.  The
	names of these tables are found in _db_tables.  The first dimension is
	for each table, the next is the entry in the table, and the third is
	the value.
	@see ConfigITest#fillDB()
	*/
  private static String[][][] _db_values = {
                                            {/* accept_list */
                                              {"'configtest1'", "'127.0.0.1'", "NULL", "0"},
                                              {"'configtest2'", "'127.0.0.1'", "NULL", "0"},
                                              {"'configtest3'", "'127.0.0.1'", "NULL", "1"}
                                            }, { /* filter */
                                              {"'testa'", "NULL", "NULL", "NULL", "NULL", "1", "1", "1", "1", "0", "0"},
                                              {"'testb'", "NULL", "NULL", "NULL", "NULL", "2", "2", "2", "1", "0", "0"},
                                              {"'testc'", "NULL", "NULL", "NULL", "NULL", "2", "2", "3", "3", "1", "1"},
                                              {"'testd'", "NULL", "NULL", "NULL", "NULL", "4", "3", "2", "1", "0", "0"},
                                            }, { /* forward_list */
                                              {"'filter1'", "'127.0.0.1'", "0"},
                                              {"'filter2'", "'127.1.0.1'", "1"}
                                            }, { /* kill_file */
                                              {"'start1'", "'end1'", "0"},
                                              {"'start2'", "'end2'", "1"},
                                              {"'start3'", "'end3'", "0"}
                                            }, { /* levels */
                                              {"'127.1.1.0'", "NULL", "2", "0", "2", "1"},
                                              {"'127.1.0.1'", "NULL", "2", "2", "1", "0"},
                                              {"'127.0.1.1'", "NULL", "2", "4", "2", "0"},
                                              {"'127.0.0.0'", "NULL", "3", "3", "3", "0"}
                                            }, { /* subordinates */
                                              {"'" + getHost() + "'", "'" + getHost() + "'"}
                                            }
                                          };


	/**
	The column names for values in the database
	*/
  private static String[][] _db_column_names = {
                                                {/* accept_list */
                                                  "filter", "start_addr", "end_addr", "inh"
                                                }, { /* filter */
                                                  "expression", "date_start", "date_end", "time_start",
                                                   "time_end", "trust", "coop", "integ", "log", "alert", "inh"
                                                }, { /* forward_list */
                                                  "filter", "host", "inh"
                                                }, { /* kill_file */
                                                  "start_addr", "end_addr", "inh"
                                                }, { /* levels */
                                                  "start_addr", "end_addr", "trust", "coop", "integ", "inh"
                                                }, { /* subordinates */
                                                   "hum_name", "subord"
                                                }
                                              };


  public ConfigITest() throws Exception {
    super();
    _local_host = InetAddress.getLocalHost();
    String address = _local_host.getHostAddress();
    Hashtable system_properties = StaticMethods.getHummerPropertyTable(true);
    _port = ((Integer)(system_properties.get("port.CONFIG_PORT"))).intValue();
    _config_server_url = system_properties.get("jdbc.url").toString();
    _sql_driver = system_properties.get("jdbc.driver").toString();
    _sql_user = system_properties.get("jdbc.user").toString();
    _sql_password = system_properties.get("jdbc.password").toString();
    Class.forName(_sql_driver);
  }

  public ConfigITest(PrintStream std_out, PrintStream err_out) throws Exception{
   super(std_out, err_out);
    _local_host = InetAddress.getLocalHost();
    String address = _local_host.getHostAddress();
    Hashtable system_properties = StaticMethods.getHummerPropertyTable(true);
    _port = ((Integer)(system_properties.get("port.CONFIG_PORT"))).intValue();
    _config_server_url = system_properties.get("jdbc.url").toString();
    _sql_driver = system_properties.get("jdbc.driver").toString();
    _sql_user = system_properties.get("jdbc.user").toString();
    _sql_password = system_properties.get("jdbc.password").toString();
    Class.forName(_sql_driver);
  }


	/**
	Allows testing for DefaultClientSocket to be run from the command line.
	If arguments are sent, the first is a filename for standard out, and the second is
	the filename for error out (one, two or no parameters will work).
	*/
  public static void main(String[] argv) throws Exception {
    PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new PrintStream(new FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new PrintStream(new FileOutputStream(argv[1]));
    }
    Test test_object = new ConfigITest(std_out, err_out);
    Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }


  protected void setTestClassName() {
    this._test_class_name = "ConfigI";
  }



  protected void testDefinition() throws Exception {
    Thread server_thread = new Thread(new ConfigITest());
    server_thread.start();
    ConfigI configi_object = null;

	/* ***************************
	Constructor testing
	*****************************/
    newMethod("Constructor");
    configi_object = (ConfigI)(runConstructorTest("No parameters", null, false)[2]);
	/* ***************************
	openServer testing
	*****************************/
    newMethod("openServer");
    runSimpleTest("Have not connected to server: Calling ConfigI.pushAccept()", "boolean false",
                  !configi_object.pushAccept()); 
    runSimpleTest("Have not connected to server: Calling ConfigI.pushFilters()", "boolean false",
                  !configi_object.pushFilters()); 
    runSimpleTest("Have not connected to server: Calling ConfigI.pushFoward()", "boolean false",
                  !configi_object.pushForward()); 
    runSimpleTest("Have not connected to server: Calling ConfigI.pushKill()", "boolean false",
                  !configi_object.pushKill()); 
    runSimpleTest("Have not connected to server: Calling ConfigI.pushLevels()", "boolean false",
                  !configi_object.pushLevels()); 
    configi_object = new ConfigI();
    InetAddress address = InetAddress.getLocalHost();
    runExceptionTest("Local host InetAddress (invalid address is responsiblity of java.io.Socket constructor)",
                     configi_object, "openServer", null, false, address);
    clearDB();

    Thread.sleep(100);
    newMethod("pushAccept (empty table)");
    runExceptionTest("Client:pushAccept with no parameters", configi_object, "pushAccept", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);
    
    Thread.sleep(100);
    newMethod("pushFilters (empty table)");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushFilter with no parameters", configi_object, "pushFilters", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);

    Thread.sleep(100);
    newMethod("pushForward (empty table)");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushForward with no parameters", configi_object, "pushForward", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(100);
    newMethod("pushKill (empty table)");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushKill with no parameters", configi_object, "pushKill", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(100);
    newMethod("pushLevels (empty table)");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushLevels with no parameters", configi_object, "pushLevels", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(100);
    newMethod("pushConfig (empty tables)");
    configi_object = new ConfigI();
    runExceptionTest("Client:pushConfig with no parameters", configi_object, "pushConfig", null, false);


    fillDB();
    Thread.sleep(100);
    newMethod("pushAccept");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushAccept with no parameters", configi_object, "pushAccept", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);

    Thread.sleep(100);
    newMethod("pushFilters");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushFilter with no parameters", configi_object, "pushFilters", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);

    Thread.sleep(100);
    newMethod("pushForward");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushForward with no parameters", configi_object, "pushForward", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(100);
    newMethod("pushKill");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushKill with no parameters", configi_object, "pushKill", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(100);
    newMethod("pushLevels");
    configi_object = new ConfigI();
    configi_object.openServer(InetAddress.getLocalHost());
    runExceptionTest("Client:pushLevels with no parameters", configi_object, "pushLevels", null, false);
    runExceptionTest("Client:closeServer with no parameters (sending data to server)",
                     configi_object, "closeServer", null, false);


    Thread.sleep(300);
    newMethod("pushConfig");
    configi_object = new ConfigI();
    runExceptionTest("Client:pushLevels with no parameters", configi_object, "pushConfig", null, false);
	/*
	Wait for the server to finish before continuing.
	*/
    server_thread.join();
  }

  public void run() {
    try {
      ServerSocket server_socket = new ServerSocket(_port);
      ObjectInputStream in_stream;
      ObjectOutputStream out_stream;
      Config input;
      Socket client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());

		/* **************************************************
		 pushAccept (empty table) testing
		 ****************************************************/

      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);

/*
      println("Server: recieved values:", false);
      for(Enumeration enum = input.data.elements(); enum.hasMoreElements();) {
        println("\t" + (String)(enum.nextElement()), false);
      }
      println("Server: correct values:", false);
      for(Enumeration enum = _correct_values[_ACCEPT_LIST].elements(); enum.hasMoreElements();) {
        println("\t" + (String)(enum.nextElement()), false);
      }
*/

      runSimpleTest("Server:After calling pushAccept and then closeServer",
                    "Recieved exactly the inherited values from the accept_list table. inh=1",
                    input.data.equals(_correct_values[_ACCEPT_LIST]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();

		/* *********************************************
		 pushFilters (empty table) testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushFilters and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_FILTER]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();

      
		/* *********************************************
		 pushForward (empty table) testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushForward and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_FORWARD_LIST]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();
      

		/* *********************************************
		 pushKill (empty table) testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushKills and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_KILL_FILE]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();
      

		/* *********************************************
		 pushLevels (empty table) testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushLevels and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_LEVELS]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();


		/* *********************************************
		 pushConfig (empty table) testing
		 **********************************************/
      Vector config_ans = new Vector();
	/*
	Set the correct values that should be recieved from pushConfig
	*/
      for(Enumeration enum = _correct_values[_FILTER].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_LEVELS].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_ACCEPT_LIST].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_FORWARD_LIST].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_KILL_FILE].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());


      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushConfig and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1. " +
                    "Assuming order is filter,levels,accept,forward,kill.",
                    input.data.equals(config_ans));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();



		/* **************************************************
		 pushAccept testing
		 ****************************************************/

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);


      runSimpleTest("Server:After calling pushAccept and then closeServer",
                    "Recieved exactly the inherited values from the accept_list table. inh=1",
                    input.data.equals(_correct_values[_ACCEPT_LIST]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();


		/* *********************************************
		 pushFilters testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushFilters and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_FILTER]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();

      
		/* *********************************************
		 pushForward testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushForward and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_FORWARD_LIST]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();
      

		/* *********************************************
		 pushKill testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushKills and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_KILL_FILE]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();
      

		/* *********************************************
		 pushLevels testing
		 **********************************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushLevels and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1",
                    input.data.equals(_correct_values[_LEVELS]));
      out_stream.close();
      in_stream.close();
      client_handler_socket.close();


		/* *********************************************
		 pushConfig testing
		 **********************************************/
      config_ans = new Vector();
	/*
	Set the correct values that should be recieved from pushConfig
	*/
      for(Enumeration enum = _correct_values[_FILTER].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_LEVELS].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_ACCEPT_LIST].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_FORWARD_LIST].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      for(Enumeration enum = _correct_values[_KILL_FILE].elements(); enum.hasMoreElements();) {
        config_ans.add(enum.nextElement());
      }
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Config)(in_stream.readObject());

      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      out_stream.writeObject(input);
      runSimpleTest("Server:After calling pushConfig and then closeServer",
                    "Recieved exactly the inherited values from the filter table. inh=1. " +
                    "Assuming order is filter,levels,accept,forward,kill.",
                    input.data.equals(config_ans));

      out_stream.close();
      in_stream.close();
      client_handler_socket.close();
    }
    catch(Exception e) {
      errorPrintln("Server: Exception stack trace: ");
      e.printStackTrace(_error_out);
      System.exit(-1);
    }
  }

  private static void clearDB() throws Exception {
    Connection connection = DriverManager.getConnection(_config_server_url, _sql_user, _sql_password);
    Statement statement = connection.createStatement();
    String query = "";
    for(int i = 0; i < _db_tables.length; i++) {
      query += "delete from " + _db_tables[i] + ";";
      _correct_values[i] = new Vector();
      _correct_values[i].add(_db_tables[i] + " delete 1");
    }
    query += "insert into subordinates values('" + getHost() + "','" + getHost() + "');";
    statement.executeUpdate(query);
    statement.close();
    connection.close();
  }


  private static void fillDB() throws Exception {
    if(_db_tables.length != _db_values.length || _db_tables.length != _db_column_names.length) {
      println("Invalid entry lengths for DB value arrays." +
              "\n_db_tables: " + _db_tables.length + "\t_db_values: " + _db_values.length +
             "\t_db_column_names: " + _db_column_names.length, true);
      System.exit(-1);
    }
    Connection connection = DriverManager.getConnection(_config_server_url, _sql_user, _sql_password);
    Statement statement = connection.createStatement();
	/*
	Because the subordinates were added during clearDB, they must be removed,
	before adding them again..having them twice is bad (because each connection is handled manually)
	*/
    String query_string = "delete from subordinates;";
    for(int i = 0; i < _db_tables.length; i++) {
      _correct_values[i] = new Vector();
      _correct_values[i].add(_db_tables[i] + " delete 1");
      for(int j = 0; j < _db_values[i].length; j++) {
        query_string += "insert into " + _db_tables[i] + " values(";
        String correct_value_string = _db_tables[i] + " insert ";
        if(_db_values[i][j].length > 0) {
          query_string += _db_values[i][j][0];
          correct_value_string += _db_values[i][j][0] + " ";
          for(int k = 1; k < _db_values[i][j].length - 1; k++) {
            query_string += "," + _db_values[i][j][k];
            correct_value_string += _db_values[i][j][k] + " ";
          }
          query_string += "," + _db_values[i][j][_db_values[i][j].length - 1];
        }
        query_string += ");";
        correct_value_string += "1";
        if(_db_values[i][j].length > 0 && !_db_values[i][j][_db_values[i][j].length - 1].equals("1")) {
          _correct_values[i].add(correct_value_string);
        }
      }
    }
    statement.executeUpdate(query_string);
  }

  private static String getHost() {
    String return_value = null;
    try {
      return_value = InetAddress.getLocalHost().getHostAddress();
    }
    catch(UnknownHostException e) {}
    return return_value;
  }
}
