/*
Java Hummer Project
File: Test.java
Date: 2-23-99
Author: Chad Ward
Description:  Test class for the MessageI class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.client;

import java.util.*;
import java.net.*;
import java.io.*;
import edu.uidaho.hummer.client.MessageI;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.test.Test;
import edu.uidaho.hummer.util.StaticMethods;

/**
Testing class for MessageI class.
*/
public class MessageITest extends Test implements Runnable {
	/**
	A message that will be used for the final testing of sendToHummer.  This value
	is static because there is an object for the server, and an object for the client, and
	they both have to have this object.
	*/
  private static Message _message = new Message();

	/**
	Properties needed for testing of sendToHummer.  This value
	is static because there is an object for the server, and an object for the client, and
	they both have to have this object.
	*/
  private static Properties _constants = new Properties(System.getProperties());


	/**
	Main method so that this test can be run from the command line
	*/
  public static void main(String[] argv) throws Exception {
    Test test_object = new MessageITest();
    test_object.runClassTest(1);
  }
	/**
	Must define this class so that the name of the class that is being tested will
	be known.
	*/
  protected void setTestClassName() {
    this._test_class_name = "MessageI";
  }


	/**
	This method defines what the testing actually does.
	Order of methods tested:
	<ul>
	  <li>Constructors
	  <li>parseParameters
	  <li>validateParameters
	  <li>sendToHummer
	</ul>
	@see edu.uidaho.hummer.client.MessageI
	*/
  protected void testDefinition() throws Exception {
    MessageI MessageI_object;

	/*
	The _constants value is not used till the end, but it was put here
	just so it doesn't clutter up the testing code.
	*/
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("MessageITest: Hummer.properties file not found.");
      fnfe.printStackTrace();
    }
    catch (IOException ioe) {
      System.err.println("MessageI: File Hummer.properties not valid.");
      ioe.printStackTrace();
    }
		/* ****************
		Constructor Tests
		******************/
		/*
		Output message saying that testing of the Constructor methods
		will be tested.
		*/
    newMethod("Constructor");

		/*
		Test that a FileNotFoundException is not thrown when calling the constructor.
		The object created by the constructor is returned, and given to MessageI_object.
		*/
    MessageI_object = (MessageI)(runConstructorTest("Did not throw BufferedInputStreamException",
                           "java.io.FileNotFoundException", false)[2]);

		/*
		Constructor test is called again, but this time, the object returned is not
		held
		*/
    runConstructorTest("Empty parameters", "java.io.IOException", false);

		/* ****************
		parseParameters
		******************/

		/*
		Output message saying that parseParameters method will be tested
		*/
    newMethod("parseParameters");
    String[] args1 = null, args2 = new String[0], args3 = {""}, arg4 = new String[2];
    Message message = new Message();

		/*
		Insure that an exception will be thrown when a null array of args is given to the method
		First value is input message.
		Second value is the object that the test is being performed with.
		Third value is the name of the method that is being tested
		Fourth value is the name (String) of the exception to look for.  null means that any Exception
			should be checked for (Exception and any sub-classes of Exception).
		Fifth value is whether the exception should be thrown or not.  If false, then test passes
			when no exception is thrown.  If true, then the test only passes if the given exception
			is thrown.
		Any additional parameters are parameters that are specific to the method being tested.  In this
			case there is only one parameter for the method.
		*/
    runExceptionTest("Null array (arguments)", MessageI_object, "parseParameters", null, true, args1, message);

		/*
		runException also returns a value.  This is an array of Objects of size=2.  The first value
		in the array is a Boolean saying whether the test passed.  The second value is the Exception
		that was thrown.  If there was no exception then this is null.
		*/
    Exception caught_exception = (Exception)(runExceptionTest("Emtpy array of strings (size=0)",
                                                  MessageI_object, "parseParameters", null,
                                                  false, args2, message)[1]);

		/*
		If there wasn't an exception caught then the message object will not be null, so the values
		in it can be tested.
		*/
    if(caught_exception == null) {
		/*
		Test everything in the Message except the date, but the date that the Message object was created
		is too much trouble to get (its in milliseconds
		*/
      runSimpleTest("Empty arguments (size=0)", "Message.description == \"\"", message.description.equals(""));
      runSimpleTest("Empty arguments (size=0)", "Message.host_IP == \"\"",  message.host_IP.equals(""));
      runSimpleTest("Empty arguments (size=0)", "Message.tool == \"\"", message.tool.equals(""));
      runSimpleTest("Message age=0",  "Message.message_age == 0", message.message_age.intValue() == 0);
    }
		/*
		reset the Message object
		*/
    message = new Message();
    caught_exception = (Exception)(runExceptionTest("Empty array of strings (single empty string)",
                                        MessageI_object, "parseParameters", null, false, args3, message)[1]);
    
    if(caught_exception == null) {
		/*
		Test everything in the Message except the date, but the date that the Message object was created
		is too much trouble to get (its in milliseconds
		*/
      runSimpleTest("Empty args (single empty string)", "Message.descrption == \"\"",
                    message.description.equals(""));
      runSimpleTest("Empty args (single empty string)", "Message.host_IP == \"\"",
                     message.host_IP.equals(""));
      runSimpleTest("Empty args (single empty string)", "Message.tool == \"\"",
                    message.tool.equals(""));
      runSimpleTest("Empty args (single empty string)", "Message.message_age == 0",
                    message.message_age.intValue() == 0);
    }
    message = new Message();
    arg4[0] = "won't work";
    arg4[1] = "nothing";
    caught_exception = (Exception)(runExceptionTest("Non-valid format of parameters {\"won't work\"}", MessageI_object,
                                         "parseParameters", null, false, arg4, message)[1]);
    if(caught_exception == null) {
		/*
		Test everything in the Message except the date, but the date that the Message object was created
		is too much trouble to get (its in milliseconds
		*/
      message = new Message();
      runSimpleTest("Invalid format of message (won't work)", "false", !MessageI_object.parseParameters(arg4, message));
      runSimpleTest("Invalid format of message (won't work)", "Message.description = \"\"",
                    message.description.equals(""));
      runSimpleTest("Invalid format of message (won't work)", "Message.host_IP = \"\"",
                    message.host_IP.equals(""));
      runSimpleTest("Invalid format of message (won't work)", "Message.tool = \"\"",
                    message.tool.equals(""));
      runSimpleTest("Invalid format of message (won't work)", "Message.message_age == 0",
                    message.message_age.intValue() == 0);
    }

		/*
		Create an array of parameters where there are several instances of the same type of
		message information.  (eg.  There are 7 different times that the filename will be set)
		The last instance should be the one that is in the message
		*/
    int large_input_size = 7 * 8;
    args1 = new String[large_input_size];
    for(int i = 0; i < (large_input_size - 1); i += 8) {
      args1[i] = "f";
      args1[i + 1] = "filename." + (i + 1);
      args1[i + 2] = "h";
      args1[i + 3] = "129.101.12." + (i + 3);
      args1[i + 4] = "m";
      args1[i + 5] = "message." + (i + 5);
      args1[i + 6] = "s";
      args1[i + 7] = "tool." + (i + 7);
    }
    message = new Message();
    try {
      MessageI_object.parseParameters(args1, message);
    }
    catch(Exception e) {
      System.err.println("MessageITest:testDefinition: Caught:" + e);
      
    }
    caught_exception = (Exception)(runExceptionTest("Large size array of parameters " +
                                                    "(multiple instances of each type)",
                                                    MessageI_object, "parseParameters",
                                                    null, false, args1, message)[1]);
    if(caught_exception == null) {
      message = new Message();
      runSimpleTest("Valid input (multiple instances giving Message members different values)",
                    "true", MessageI_object.parseParameters(args1, message) == true);
      runSimpleTest("Valid input (multiple instances giving Message members different values)",
                    "Message.file equals last value in parameters for \"f\"",
                    message.filename.equals("filename." + (large_input_size - 7)));
      runSimpleTest("Valid input (multiple instances giving Message members different values)",
                    "Message.host_IP equals last value in parameters for \"h\"",
                    message.host_IP.equals("129.101.12." + (large_input_size - 5)));
      runSimpleTest("Valid input (multiple instances giving Message members different values)",
                    "Message.description equals last value in parameters for \"d\"",
                    message.description.equals("message." + (large_input_size - 3)));
      runSimpleTest("Valid input (multiple instances giving Message members different values)",
                    "Message.tool equals last value in parameters for \"s\"",
                    message.tool.equals("tool." + (large_input_size - 1)));
    }

    runExceptionTest("Null message pointer",
                     MessageI_object, "parseParameters", null, true, args3, null);

		/* ****************
		validateParameters
		******************/
    newMethod("validateParameters");
    message = new Message();
    message.description = "";
    message.host_IP = "not_an_ip";

    runSimpleTest("Message with empty description", "false", !MessageI_object.validateParameters(message));
    message.description = "This is a message description";

		/*
		For some reason MessageI does not validate that the
		hostip is valid until sendToHummer is called and an Exception is thrown, but this
		is what is supposed to happen (I guess)
		*/
    runSimpleTest("Message with invalid IP", "true", MessageI_object.validateParameters(message));

    runExceptionTest("Invalid IP", MessageI_object, "validateParameters", null, false, message);

    Boolean passed = (Boolean)(runExceptionTest("Empty IP", MessageI_object, "validateParameters",
                                                null, false, message)[0]);
    if(passed.booleanValue()) {
      runSimpleTest("Valid message object (description is not empty, host_IP is empty)", "false",
                    MessageI_object.validateParameters(message));
    }

		/* ****************
		sendToHummer
		******************/
    newMethod("sendToHummer");
    try {
		/*
		Initialize the Message object that will be sent to the mock server
		*/
      _message.host_IP = InetAddress.getLocalHost().getHostAddress();
      _message.description = "message_description";
      _message.tool = "bdf";
      _message.message_age = new Integer(2);

		/*
		Create the Thread for the mock server
		*/
      Thread server_socket_thread = new Thread(new MessageITest());
      Message bad_message = new Message();

      bad_message.description = "The bad message";
      bad_message.host_IP = "not a host ip";

		/*
		The exception for invalid IP addresses is caught inside the validateParameters method, so
		runExceptionTest will not work.  Only a test for false being returned can be done.
		*/
      runSimpleTest("Message.host_IP is not valid IP", "false", !MessageI_object.sendToHummer(bad_message));

		/*
		Start the thread for the mock server
		*/
      server_socket_thread.start();

		/*
		Send the message to the server, and test that an exception was not thrown
		in the process
		*/ 	
      String input_message = "Valid message (description:\"" + _message.description + "\" tool:\"" +
                             _message.tool + "\" age:" + _message.message_age + " host_IP:" +
                             _message.host_IP + ")";
      runSimpleTest(input_message, "true", MessageI_object.sendToHummer(_message));

		/*
		Wait for the server to finish before ending this function.  If this isn't done, then
		a report of the results will be done before they have all been calculated
		*/
      server_socket_thread.join();
    }
    catch(Exception e) {
		/*
		If anything goes wrong, then print where the exception is caught, and then
		a stack trace
		*/
      System.err.println("ToolTestI:testDefinition:");
      e.printStackTrace();
    }
  }

	/**
	A thread is used to represent a Hummer tool server.  Will retrieve the message
	object and test the values for consistency
	*/
  public void run() {
    try {
		/*
		Get the port number from the Hummer.properties file
		*/
      int port_number = (new Integer(_constants.getProperty("port.MESSAGE_PORT"))).intValue();

		/*
		Create a ServerSocket on the correct port, and wait for the client to connect
		Hopefully there isn't a Hummer already running on this machine.
		*/
      ServerSocket server_socket = new ServerSocket(port_number);
      Socket client_socket = server_socket.accept();

		/*
		Read in the message from the client and test it against the values
		that it should be (_message holds this information)
		*/
      ObjectInputStream input_stream = new ObjectInputStream(client_socket.getInputStream());
      Message read_message = (Message)(input_stream.readObject());

		/*
		Wait a bit, otherwise some race conditions mess up the output
		*/
  //    Thread.sleep(50);

		/*
		Run the test
		*/
      runSimpleTest("Message recieved from socket", "Message recieved equal to message sent (age=0)",
                    _message.host_IP.equals(read_message.host_IP) &&
                    _message.description.equals(read_message.description) &&
                    _message.tool.equals(read_message.tool) &&
                    read_message.message_age.intValue() == 0);
    }
    catch(Exception e) {
      System.err.println("MessageITest:run:");
      e.printStackTrace();
    }
  }
}