/*
Java Hummer Project
File: Test.java
Date: 2-17-99
Author: Chad Ward
Description:  Parent class for all tests run.  Classes extending
              This class only have to define the testDefinition method,
              and then use the runTest method to perform the test.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.client;

import java.io.PrintStream;
import java.net.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import edu.uidaho.hummer.test.Test;
import edu.uidaho.hummer.client.PeerI;
import edu.uidaho.hummer.dataobject.Peer;
import edu.uidaho.hummer.util.StaticMethods;

/**
Test class for client.PeerI
@author Chad Ward
*/
public class PeerITest extends Test implements Runnable {

  private static int _port;
  private static String _peer_server_url, _sql_driver, _sql_user, _sql_password;
  private static Properties _constants = System.getProperties();
  private static InetAddress _local_host;
	/**
	List of table names.  The values for these tables are found in
	_db_values.
	@see PeerITest#fillDB()
	*/
  private static final String[] _db_tables = { "pgroups"};

  private static final int _PGROUPS = 0;

	/**
	Correct values for accept_list test
	*/
  private static Vector[] _correct_values = new Vector[_db_tables.length];
  
	/**
	The values to be placed in the tables of the database.  The
	names of these tables are found in _db_tables.  The first dimension is
	for each table, the next is the entry in the table, and the third is
	the value.
	@see PeerITest#fillDB()
	*/
  private static String[][][] _db_values = {
                                            {/* pgroups */
                                              {"'peergroup1a'", "'" + getHost().getHostAddress() + "'",
                                               "'billybob@uidaho.edu'"},
                                              {"'peergroup2a'", "'" + getHost().getHostAddress() + "'", "NULL"},
                                            }
                                          };


	/**
	The column names for values in the database
	*/
  private static String[][] _db_column_names = {
                                                {/* pgroups */
                                                  "pgroup", "mod_addr", "email"
                                                }
                                              };


  public PeerITest() throws Exception{
    super();
    getHost();
    String address = _local_host.getHostAddress();
    Hashtable system_properties = StaticMethods.getHummerPropertyTable(true);
    _port = ((Integer)(system_properties.get("port.PEER_PORT"))).intValue();
    _peer_server_url = system_properties.get("jdbc.url").toString();
    _sql_driver = system_properties.get("jdbc.driver").toString();
    _sql_user = system_properties.get("jdbc.user").toString();
    _sql_password = system_properties.get("jdbc.password").toString();
    Class.forName(_sql_driver);
    fillDB();
  }

  public PeerITest(PrintStream std_out, PrintStream err_out) throws Exception {
    super(std_out, err_out);
    getHost();
    String address = _local_host.getHostAddress();
    Hashtable system_properties = StaticMethods.getHummerPropertyTable(true);
    _port = ((Integer)(system_properties.get("port.PEER_PORT"))).intValue();
    _peer_server_url = system_properties.get("jdbc.url").toString();
    _sql_driver = system_properties.get("jdbc.driver").toString();
    _sql_user = system_properties.get("jdbc.user").toString();
    _sql_password = system_properties.get("jdbc.password").toString();
    Class.forName(_sql_driver);
    fillDB();
  }


	/**
	Allows testing for DefaultClientSocket to be run from the command line.
	If arguments are sent, the first is a filename for standard out, and the second is
	the filename for error out (one, two or no parameters will work).
	*/
  public static void main(String[] argv) throws Exception {
    PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new PrintStream(new FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new PrintStream(new FileOutputStream(argv[1]));
    }
    Test test_object = new PeerITest(std_out, err_out);
    Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }


  protected void setTestClassName() {
    this._test_class_name = "PeerI";
  }



  protected void testDefinition() throws Exception {
    Thread server_thread = new Thread(new PeerITest());
    server_thread.start();

	/**
	Clean up the database a little bit before beginning...this test should not be run
	on a system where the database is needed.
	*/

    PeerI peeri_object = null;
//    System.setOut(new PrintStream(new FileOutputStream("testee_class.out")));
	/* ***************************
	Constructor testing
	*****************************/
    newMethod("Constructor");
    peeri_object = (PeerI)(runConstructorTest("No parameters", null, false)[2]);
    if(peeri_object == null) {
      return;
    }

	/* ***************************
	getModeratorAddress testing
	*****************************/
    newMethod("getModeratorAddress");
    InetAddress address = (InetAddress)(
           runExceptionTest("Client: Invalid group name (unknownname)", peeri_object, "getModeratorAddress",
                            null, false, "unknownname")[2]);
    runSimpleTest("Client: Invalid group name (unknownname)", "Null address", address == null);
    address = (InetAddress)(
           runExceptionTest("Client: Valid group name (peergroup1a)", peeri_object, "getModeratorAddress",
                            null, false, "peergroup1a")[2]);
    if(runSimpleTest("Client: Valid group name(peergroup1a)", "Non-null value", address != null)) {
      runSimpleTest("Client: Valid group name(peergroup1a)", "Local address (" + _local_host.getHostAddress() + ")",
                    address.equals(InetAddress.getLocalHost()));
    }
    address = (InetAddress)(
           runExceptionTest("Client: A second valid group name (peergroup2a)", peeri_object, "getModeratorAddress",
                     null, false, "peergroup2a")[2]);
    if(runSimpleTest("Client: Valid group name(peergroup2a)", "Non-null value", address != null)) {
      runSimpleTest("Client: Valid group name(peergroup2a)", "Local address (" + _local_host.getHostAddress() + ")",
                    address.equals(InetAddress.getLocalHost()));
    }

    newMethod("sendMessageToGroup");
    runExceptionTest("Client: Unknown group name (unknowngroup)", peeri_object,
                                    "sendMessageToGroup", "java.lang.NullPointerException", true, "unknowngroup", "hello group");

    runExceptionTest("Client: group equals peergroup1a and message equals: \"hello group\"",
                     peeri_object, "sendMessageToGroup", null, false, "peergroup1a", "hello group");

    runExceptionTest("Client: group equals peergroup2a and message equals: \"hello group\"",
                     peeri_object, "sendMessageToGroup", null, false, "peergroup2a", "hello group");

	/* ***************************
	subscribeToGroup testing
	*****************************/
    Thread.sleep(100);
    newMethod("subscribeToGroup");
    Vector correct_pgroup_table = StaticMethods.getDBTable("pgroups", null),
           correct_peer_table = StaticMethods.getDBTable("peers", null),
           correct_level_table = StaticMethods.getDBTable("levels", null);
    Hashtable pg_first = new Hashtable(),
              pg_second = new Hashtable(),
              pg_third = new Hashtable(),
              peer_first = new Hashtable(),
              peer_second = new Hashtable(),
              peer_third = new Hashtable(),
              level_first = new Hashtable(),
              level_second = new Hashtable(),
              level_third = new Hashtable();

    pg_first.put("pgroup", "unknowngroup");
    pg_first.put("mod_addr", getHost().getHostAddress());
    pg_first.put("email", "NULL");
    peer_first.put("pgroup", "unknowngroup");
    peer_first.put("mem_addr", "addr1");
    peer_first.put("peer_sub", "1");
    peer_first.put("peer_mod", "1");
    level_first.put("start_addr", "129.101.9.12");
    level_first.put("end_addr", "129.101.9.6");   
    level_first.put("trust", "3");
    level_first.put("coop", "5"); 
    level_first.put("integ", "3");
    level_first.put("inh", "1");  

    pg_second.put("pgroup", "peergroup1");
    pg_second.put("mod_addr", getHost().getHostAddress());
    pg_second.put("email", "billy@bob.edu");
    peer_second.put("pgroup", "peergroup1");
    peer_second.put("mem_addr", getHost().getHostAddress());
    peer_second.put("peer_sub", "1");
    peer_second.put("peer_mod", "0");
    level_second.put("start_addr", "129.102.9.4");
    level_second.put("end_addr", "129.102.9.3");   
    level_second.put("trust", "2");
    level_second.put("coop", "2"); 
    level_second.put("integ", "2");
    level_second.put("inh", "1");  

    pg_third.put("pgroup", "peergroup2");
    pg_third.put("mod_addr", getHost().getHostAddress());
    pg_third.put("email", "billy@bob2.edu");
    peer_third.put("pgroup", "peergroup2");
    peer_third.put("mem_addr", getHost().getHostAddress());
    peer_third.put("peer_sub", "0");
    peer_third.put("peer_mod", "1");
    level_third.put("start_addr", "129.101.9.2");
    level_third.put("end_addr", "129.101.9.1");   
    level_third.put("trust", "1");
    level_third.put("coop", "1"); 
    level_third.put("integ", "1");
    level_third.put("inh", "1");  

    Boolean return_value;
    Vector pg_table = new Vector(),
           peer_table = new Vector(),
           level_table = new Vector();

    return_value = (Boolean)(runExceptionTest("Client: Unknown group name (unknowngroup)", peeri_object,
                                              "subscribeToGroup", null, false, "unknowngroup", _local_host)[2]);
    if(runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.add(pg_first);
      correct_peer_table.add(peer_first);
      correct_level_table.add(level_first);
      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);
//      println("Correct table values:\n" + tableToString(correct_pgroup_table) +
//              "\nDB values:\n" + tableToString(db_table), true);
      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly (used sql sent by server)",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in peers table updated correctly (used sql sent by server)",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in levels table updated correctly (used sql sent by server)",
                    correct_level_table.equals(level_table));
    }


    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "subscribeToGroup", null, false,
                                              "peergroup1", _local_host)[2]);
    if(runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.add(pg_second);
      correct_peer_table.add(peer_second);
      correct_level_table.add(level_second);
      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);
//      println("Correct table values:\n" + tableToString(correct_level_table) +
//              "\nDB values:\n" + tableToString(level_table), true);
      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly (used sql sent by server)",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in peers table updated correctly (used sql sent by server)",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in levels table updated correctly (used sql sent by server)",
                    correct_level_table.equals(level_table));
    }

    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                                              peeri_object, "subscribeToGroup", null, false,
                                              "peergroup2", _local_host)[2]);
    if(runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.add(pg_third);
      correct_peer_table.add(peer_third);
      correct_level_table.add(level_third);
      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);
//      println("Correct table values:\n" + tableToString(correct_pgroup_table) +
//              "\nDB values:\n" + tableToString(db_table), true);
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly (used sql sent by server)",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in peers table updated correctly (used sql sent by server)",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in levels table updated correctly (used sql sent by server)",
                    correct_level_table.equals(level_table));
    }

    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "subscribeToGroup", null, false,
                                              "peergroup1", _local_host)[2]);
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "boolean false",
                  !return_value.booleanValue());
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "pgroups table has not changed",
                  correct_pgroup_table.equals(StaticMethods.getDBTable("pgroups", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "peers table has not changed",
                  correct_peer_table.equals(StaticMethods.getDBTable("peers", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "levels table has not changed",
                  correct_level_table.equals(StaticMethods.getDBTable("levels", null)));

    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "subscribeToGroup", null, false,
                                              "peergroup1", _local_host)[2]);
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent message for Peer.action)", "boolean false",
                  !return_value.booleanValue());
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "pgroups table has not changed",
                  correct_pgroup_table.equals(StaticMethods.getDBTable("pgroups", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "peers table has not changed",
                  correct_peer_table.equals(StaticMethods.getDBTable("peers", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "levels table has not changed",
                  correct_level_table.equals(StaticMethods.getDBTable("levels", null)));




	/* ***************************
	unsubscribeFromGroup testing
	*****************************/
    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "unsubscribeFromGroup", null, false,
                                              "peergroup1")[2]);
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "boolean false",
                  !return_value.booleanValue());
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "pgroups table has not changed",
                  correct_pgroup_table.equals(StaticMethods.getDBTable("pgroups", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "peers table has not changed",
                  correct_peer_table.equals(StaticMethods.getDBTable("peers", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server placed invalid date in Peer object)", "levels table has not changed",
                  correct_level_table.equals(StaticMethods.getDBTable("levels", null)));

    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "unsubscribeFromGroup", null, false,
                                              "peergroup1")[2]);
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent message for Peer.action)", "boolean false",
                  !return_value.booleanValue());
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "pgroups table has not changed",
                  correct_pgroup_table.equals(StaticMethods.getDBTable("pgroups", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "peers table has not changed",
                  correct_peer_table.equals(StaticMethods.getDBTable("peers", null)));
    runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\" " +
                  "(Server sent \"message\" for action)", "levels table has not changed",
                  correct_level_table.equals(StaticMethods.getDBTable("levels", null)));


    return_value = (Boolean)(runExceptionTest("Client: Unknown group name (unknowngroup)", peeri_object,
                                              "unsubscribeFromGroup", null, false, "unknowngroup")[2]);
    if(runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.remove(correct_pgroup_table.size() - 3);
      correct_peer_table.remove(correct_peer_table.size() - 3);

      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);

      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in peers table updated correctly",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals unknowngroup and message equals: \"hello group\"",
                    "Values in levels table have not changed",
                    correct_level_table.equals(level_table));
    }


    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                                              peeri_object, "unsubscribeFromGroup", null, false,
                                              "peergroup1")[2]);
    if(runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.remove(correct_pgroup_table.size() - 2);
      correct_peer_table.remove(correct_peer_table.size() - 2);

      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);
//      println("Correct table values:\n" + tableToString(correct_pgroup_table) +
//              "\nDB values:\n" + tableToString(pg_table), true);

      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly (used sql sent by server)",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in peers table updated correctly (used sql sent by server)",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals peergroup1 and message equals: \"hello group\"",
                    "Values in levels table have not changed",
                    correct_level_table.equals(level_table));
    }

    return_value = (Boolean)(runExceptionTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                                              peeri_object, "unsubscribeFromGroup", null, false,
                                              "peergroup2")[2]);
    if(runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"", "boolean true",
                     return_value.booleanValue())) {
      correct_pgroup_table.remove(correct_pgroup_table.size() - 1);
      correct_peer_table.remove(correct_peer_table.size() - 1);

      pg_table = StaticMethods.getDBTable("pgroups", null);
      peer_table = StaticMethods.getDBTable("peers", null);
      level_table = StaticMethods.getDBTable("levels", null);
//      println("Correct table values:\n" + tableToString(correct_pgroup_table) +
//              "\nDB values:\n" + tableToString(db_table), true);
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in pgroups table updated correctly (used sql sent by server)",
                    correct_pgroup_table.equals(pg_table));
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in peers table updated correctly (used sql sent by server)",
                    correct_peer_table.equals(peer_table));
      runSimpleTest("Client: group equals peergroup2 and message equals: \"hello group\"",
                    "Values in levels table have not changed",
                    correct_level_table.equals(level_table));
    }

	/*
	Wait for the server to finish before continuing.
	*/
    server_thread.join();
  }

  public void run() {
    try {
      ServerSocket server_socket = new ServerSocket(_port);
      ObjectInputStream in_stream;
      ObjectOutputStream out_stream;
      Peer input;
      Socket client_handler_socket = null;

	/* ***************************
	getModeratorAddress testing
	*****************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.action equals \"message\"", input.action.equals("message"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup1a\"", input.peer_group.equals("peergroup1a"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.message.host_IP equals \"" + InetAddress.getLocalHost().getHostAddress() + "\"",
                      input.message.host_IP.equals(InetAddress.getLocalHost().getHostAddress()));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.message.description equals \"hello group\"",
                      input.message.description.equals("hello group"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.message.filename equals \"peergroup1a\"",
                      input.message.filename.equals("peergroup1a"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.message.tool equals \"peers\"", input.message.tool.equals("peers"));
      }

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.action equals \"message\"", input.action.equals("message"));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup2a\"", input.peer_group.equals("peergroup2a"));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.message.host_IP equals \"" + InetAddress.getLocalHost().getHostAddress() + "\"",
                      input.message.host_IP.equals(InetAddress.getLocalHost().getHostAddress()));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.message.description equals \"hello group\"",
                      input.message.description.equals("hello group"));
        runSimpleTest("Server: After client sent peergroup2a group name and message equal to \"hello group\"",
                      "Peer.message.filename equals \"peergroup2a\"",
                      input.message.filename.equals("peergroup2a"));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.message.tool equals \"peers\"", input.message.tool.equals("peers"));
      }


	/* ***************************
	subscribeToGroup testing
	*****************************/
      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                      "Peer.action equals \"subscribe\"", input.action.equals("subscribe"));
        runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"unknowngroup\"", input.peer_group.equals("unknowngroup"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      Peer output = new Peer();
      output.date = input.date;
      output.action = "subscribed";
      output.statements.add("insert into pgroups values('unknowngroup', '" + getHost().getHostAddress() + "', NULL)");
      output.statements.add("insert into peers values('unknowngroup', 'addr1', 1, 1)");
      output.statements.add("insert into levels values('129.101.9.12', '129.101.9.6', 3, 5, 3, 1)");
      out_stream.writeObject(output);
      client_handler_socket.close();

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.action equals \"subscribe\"", input.action.equals("subscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "subscribed";
      output.statements.add("insert into pgroups values('peergroup1', '" +
                            getHost().getHostAddress() + "', 'billy@bob.edu')");
      output.statements.add("insert into peers values('peergroup1', '" + getHost().getHostAddress() + "', 1, 0)");
      output.statements.add("insert into levels values('129.102.9.4', '129.102.9.3', 2, 2, 2, 1)");
      out_stream.writeObject(output);
      client_handler_socket.close();


      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.action equals \"subscribe\"", input.action.equals("subscribe"));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup2\"", input.peer_group.equals("peergroup2"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "subscribed";
      output.statements.add("insert into pgroups values('peergroup2', '" +
                            getHost().getHostAddress() + "', 'billy@bob2.edu')");
      output.statements.add("insert into peers values('peergroup2', '" + getHost().getHostAddress() + "', 0, 1)");
      output.statements.add("insert into levels values('129.101.9.2', '129.101.9.1', 1, 1, 1, 1)");
      out_stream.writeObject(output);


      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and " +
                       "message equal to \"hello group\" (sending invalid date back to client)",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending invalid date back to client)",
                      "Peer.action equals \"subscribe\"", input.action.equals("subscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending invalid date back to client)",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = new java.util.Date();
      output.action = "subscribed";

      output.statements.add("insert into pgroups values('peergroup1', '" +
                            getHost().getHostAddress() + "', 'billy@bob.edu')");
      output.statements.add("insert into peers values('peergroup1', '" + getHost().getHostAddress() + "', 1, 0)");
      output.statements.add("insert into levels values('129.102.9.4', '129.102.9.3', 2, 2, 2, 1)");
      out_stream.writeObject(output);
      client_handler_socket.close();


      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and " +
                       "message equal to \"hello group\" (sending action equal to message)",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending action equal to message)",
                      "Peer.action equals \"subscribe\"", input.action.equals("subscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending action equal to message)",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "message";
      output.statements.add("insert into pgroups values('peergroup1', '" +
                            getHost().getHostAddress() + "', 'billy@bob.edu')");
      output.statements.add("insert into peers values('peergroup1', '" + getHost().getHostAddress() + "', 1, 0)");
      output.statements.add("insert into levels values('129.102.9.4', '129.102.9.3', 2, 2, 2, 1)");
      out_stream.writeObject(output);
      client_handler_socket.close();



	/* ***************************
	unsubscribeFromGroup testing
	*****************************/

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and " +
                       "message equal to \"hello group\" (sending invalid date back to client)",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending invalid date back to client)",
                      "Peer.action equals \"unsubscribe\"", input.action.equals("unsubscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending invalid date back to client)",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = new java.util.Date();
      output.action = "unsubscribed";
      out_stream.writeObject(output);
      client_handler_socket.close();


      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and " +
                       "message equal to \"hello group\" (sending action equal to \"message\")",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending action equal to \"message\")",
                      "Peer.action equals \"unsubscribe\"", input.action.equals("unsubscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and " +
                      "message equal to \"hello group\" (sending action equal to \"message\")",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "message";
      out_stream.writeObject(output);
      client_handler_socket.close();

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                      "Peer.action equals \"unsubscribe\"", input.action.equals("unsubscribe"));
        runSimpleTest("Server: After client sent unknowngroup group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"unknowngroup\"", input.peer_group.equals("unknowngroup"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "unsubscribed";
      out_stream.writeObject(output);
      client_handler_socket.close();

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.action equals \"unsubscribe\"", input.action.equals("unsubscribe"));
        runSimpleTest("Server: After client sent peergroup1 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup1\"", input.peer_group.equals("peergroup1"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "unsubscribed";
      out_stream.writeObject(output);
      client_handler_socket.close();

      client_handler_socket = server_socket.accept();
      in_stream = new ObjectInputStream(client_handler_socket.getInputStream());
      input = (Peer)(in_stream.readObject());
      if(runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                       "Non-null Peer object recieved", input != null)) {
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.action equals \"unsubscribe\"", input.action.equals("unsubscribe"));
        runSimpleTest("Server: After client sent peergroup2 group name and message equal to \"hello group\"",
                      "Peer.peer_group equals \"peergroup2\"", input.peer_group.equals("peergroup2"));
      }
      out_stream = new ObjectOutputStream(client_handler_socket.getOutputStream());
      output = new Peer();
      output.date = input.date;
      output.action = "unsubscribed";
      out_stream.writeObject(output);


    }
    catch(Exception e) {
      System.err.println("Server: Exception stack trace: ");
      e.printStackTrace(_error_out);
      System.exit(-1);
    }
  }


  private static void fillDB() throws Exception {
    if(_db_tables.length != _db_values.length || _db_tables.length != _db_column_names.length) {
      System.err.println("Invalid entry lengths for DB value arrays." +
                         "\n_db_tables: " + _db_tables.length + "\t_db_values: " + _db_values.length +
                         "\t_db_column_names: " + _db_column_names.length);
      System.exit(-1);
    }
    String final_query_string = "";
    Connection connection = DriverManager.getConnection(_peer_server_url, _sql_user, _sql_password);
    Statement statement = connection.createStatement();

    final_query_string ="delete from pgroups; delete from peers; delete from levels;";
    for(int i = 0; i < _db_tables.length; i++) {
      for(int j = 0; j < _db_values[i].length; j++) {
        String query_string = "insert into " + _db_tables[i] + " values(";
        if(_db_values[i][j].length > 0) {
          query_string += _db_values[i][j][0];
          for(int k = 1; k < _db_values[i][j].length - 1; k++) {
            query_string += "," + _db_values[i][j][k];
          }
          query_string += "," + _db_values[i][j][_db_values[i][j].length - 1];
        }
        query_string += ");";
        final_query_string += query_string;
      }
    }
    statement.executeUpdate(final_query_string);
  }

  private static InetAddress getHost() {
    if(_local_host == null) {
      try {
        _local_host = InetAddress.getLocalHost();
      }
      catch(UnknownHostException e) {
        errorPrintln("PeerITest:getHost: Error while trying to retrieve local host address: " + e);
        System.exit(-1);
      }
    }
    return _local_host;
  }

	/**
	Converts value returned by getDBTable to a string with columns seperated by tabs
	*/
  private String tableToString(Vector table) {
    String return_value = "";
    Hashtable current_row;
    String current_column;
    Object current_column_object;
    for(Enumeration table_enum = table.elements(); table_enum.hasMoreElements();) {
      current_row = (Hashtable)(table_enum.nextElement());
      for(Enumeration row_enum = current_row.keys(); row_enum.hasMoreElements();) {
        current_column_object = current_row.get(row_enum.nextElement());
        return_value += current_column_object.toString() + "\t";
      }
      if(table_enum.hasMoreElements()) {
        return_value += "\n";
      }
    }
    return return_value;
  }
}
