/*
Java Hummer Project
File: HummerCertificateTest.java
Date: 2-17-99
Author: Chad Ward
Description:  Tests HummerCertificate class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.dataobject;

import java.security.*;
import java.io.*;
import java.util.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.test.Test;

/**
Tests HummerCertificate class
@author Chad Ward
*/
public class HummerCertificateTest extends Test {

        /**
        Main method so that this test can be run from the command line
        */
  public static void main(String[] argv) throws Exception {
    java.io.PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new java.io.PrintStream(new java.io.FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new java.io.PrintStream(new java.io.FileOutputStream(argv[1]));
    }
    Test test_object = new HummerCertificateTest(std_out, err_out);
    java.util.Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }

  public HummerCertificateTest() { super(); }
  public HummerCertificateTest(java.io.PrintStream out, java.io.PrintStream err) { super(out, err); }
	/**
	Initializes the _test_class_name variable.
	Grabs the name of the test class using the Class.getName() method and then removing the last 4
	characters in the name ("Test")
	*/
  protected void setTestClassName() {
    _test_class_name = "HummerCertificate";
  }

  protected void testDefinition() throws Exception {

    GregorianCalendar now = new GregorianCalendar(), expiration_date = new GregorianCalendar();
    expiration_date.add(Calendar.DAY_OF_YEAR, 5);
    Hashtable access_table = new Hashtable();
    access_table.put("value1", new Boolean(true));
    access_table.put("value2", new Boolean(false));
    KeyPair key_pair1 = StaticMethods.generateKeyPair(), key_pair2 = StaticMethods.generateKeyPair();
    Signature sign1 = StaticMethods.getSigningSignatureObject(key_pair1.getPrivate()),
              verify1 = StaticMethods.getVerifyingSignatureObject(key_pair1.getPublic()),
              sign2 = StaticMethods.getSigningSignatureObject(key_pair2.getPrivate()),
              verify2 = StaticMethods.getVerifyingSignatureObject(key_pair2.getPublic());

		/* ***********************
		Constructor testing
		*************************/
    newMethod("Constructor");
    String class_name = "edu.uidaho.hummer.dataobject.HummerCertificate";
    HummerCertificate hummer_certificate = (HummerCertificate)
            (runConstructorTest("Did not throw exception with \"acceptable\" parameters",
                               null, false, sign1,
                               key_pair2.getPublic(), access_table, expiration_date)[2]);

    runConstructorTest("Null signature_object", null, true,
                       null, key_pair2.getPublic(), access_table, now);

    runConstructorTest("Null public key", null, true,
                       sign1, null, access_table, now);

    runConstructorTest("Null access_table", null, false,
                       sign1, key_pair2.getPublic(), null, now);

    runConstructorTest("Null expiration date", null, true,
                       sign1, key_pair2.getPublic(), access_table, null);

		/* ***********************
		isValid testing
		*************************/
    newMethod("isValid");
    runSimpleTest("Non-null correct Signature", "true", hummer_certificate.isValid(verify1));
    runSimpleTest("Incorrect non-null Signature", "false",
                  !hummer_certificate.isValid(verify2));
    runExceptionTest("Null signature object", hummer_certificate, "isValid",
                     "java.lang.NullPointerException", true, null);
    ByteArrayOutputStream byte_out_stream = new ByteArrayOutputStream();
    ObjectOutputStream out_stream = new ObjectOutputStream(byte_out_stream);
    out_stream.writeObject(hummer_certificate);
    byte[] bytes = byte_out_stream.toByteArray();
    ByteArrayInputStream byte_in_stream = new ByteArrayInputStream(bytes);
    ObjectInputStream in_stream = new ObjectInputStream(byte_in_stream);
    HummerCertificate new_certificate = (HummerCertificate)(in_stream.readObject());
//    runSimpleTest("After serializing and deserializing HummerCertificate",
//                  "Equal to previous value",
//                  new_certificate.equals(hummer_certificate,
//                                         StaticMethods.getVerifyingSignatureObject(hummer_certificate.getPublicKey())));
//    runSimpleTest("After serializing and deserializing HummerCertificate",
//                  "Object contained is equal", new_certificate.getObject().equals(hummer_certificate.getObject()));
    runSimpleTest("After serializing and deserializing HummerCertificate",
                  "Still valid with correct Signature object", new_certificate.isValid(verify1));

		/* ***********************
		getPublicKey testing
		*************************/
    newMethod("getPublicKey");
    runSimpleTest("No parameters", "PublicKey equal to PublicKey given",
                  hummer_certificate.getPublicKey().equals(key_pair2.getPublic()));

		/* ***********************
		getAccessTable testing
		*************************/
    newMethod("getAccessTable");
    runSimpleTest("No parameters", "Access table to the one given",
                  hummer_certificate.getAccessTable().equals(access_table));

		/* ***********************
		getExpirationDate testing
		*************************/
    newMethod("getExpirationDate");
    runSimpleTest("No parameters", "Expiration date equal to the one given", 
                  hummer_certificate.getExpirationDate().equals(expiration_date));

		/* ***********************
		isExpired testing
		*************************/
    newMethod("isExpired");
    HummerCertificate hummer_certificate2 = new HummerCertificate(sign1, key_pair2.getPublic(), 
                                                                access_table, now);
    runSimpleTest("HummerCertificate that was given expired date", "true", hummer_certificate2.isExpired());
    runSimpleTest("HummerCertificate that will expire in about 5 days", "false",
                  !hummer_certificate.isExpired());
  }
}