/*
Java Hummer Project
File: NewUserValidatorTest.java
Date: 2-17-99
Author: Chad Ward
Description:  Tests StaticMethods class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.dataobject;

import java.security.*;
import java.util.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.test.Test;

public class NewUserValidatorTest extends Test {

        /**
        Main method so that this test can be run from the command line
        */
  public static void main(String[] argv) throws Exception {
    Test test_object = new NewUserValidatorTest();
    test_object.runClassTest(1);
  }

	/**
	Initializes the _test_class_name variable.
	Grabs the name of the test class using the Class.getName() method and then removing the last 4
	characters in the name ("Test")
	*/
  protected void setTestClassName() {
    _test_class_name = "NewUserValidator";
  }

  protected void testDefinition() throws Exception {
    KeyPair key_pair = StaticMethods.generateKeyPair();
    byte[] password = "test".getBytes(), wrong_password = "wrong".getBytes();
    byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(), 
           null_password_digest = StaticMethods.getDigest(null, random_timestamp, key_pair.getPublic()),
           null_random_digest = StaticMethods.getDigest(password, null, key_pair.getPublic()),
           null_both_digest = StaticMethods.getDigest(null, null, key_pair.getPublic()),
           digest = StaticMethods.getDigest(password, random_timestamp, key_pair.getPublic()),
           wrong_random_timestamp = StaticMethods.getRandomTimestampBytes(),
           wrong_password_digest = StaticMethods.getDigest(wrong_password, random_timestamp, key_pair.getPublic()),
           wrong_timestamp_digest = StaticMethods.getDigest(password, wrong_random_timestamp, key_pair.getPublic());

		/* ***********************
		Constructor testing
		*************************/
    newMethod("Constructor");
    String class_name = "edu.uidaho.hummer.dataobject.NewUserValidator";
    NewUserValidator new_user_null_password, new_user;
    new_user_null_password = (NewUserValidator)(runConstructorTest("Did not throw exception for null password",
                                                null, false, (String)null)[2]);
    new_user = (NewUserValidator)(runConstructorTest("Did not throw exception for non-null password",
                                                null, false, password)[2]);

		/* ***********************
		isValidPassword testing
		*************************/
    newMethod("isValidPassword");
    runSimpleTest("Non-null password and non-null random-timestamp", "true", 
                 new_user.isValidPassword(digest, random_timestamp, key_pair.getPublic()));
    runSimpleTest("Non-null password and null random-timestamp", "true",
                 new_user.isValidPassword(null_random_digest, null, key_pair.getPublic()));
    runSimpleTest("Null password and non-null random-timestamp (user initialize with null password)", "true",
                 new_user_null_password.isValidPassword(null_password_digest, random_timestamp,
                                                        key_pair.getPublic()));
    runSimpleTest("Null password and null random-timestamp (user initialize with null password)", "true",
                 new_user_null_password.isValidPassword(null_both_digest, null,
                                                        key_pair.getPublic()));
    runSimpleTest("Wrong password for non-null password and non-null random-timestamp", "false",
                 !new_user.isValidPassword(wrong_password_digest, random_timestamp,
                                           key_pair.getPublic()));
    runSimpleTest("Wrong random-timestamp for non-null password and non-null random-timestamp", "false",
                 !new_user.isValidPassword(wrong_password_digest, wrong_random_timestamp,
                                           key_pair.getPublic()));

		/* ***********************
		clear testing
		*************************/
    newMethod("clear");
    runExceptionTest("No parameters", new_user, "clear", null, false);
  }
}
