/*
Java Hummer Project
File: ConfigServerTest.java
Date: 2-15-99
Author: Joel Marlow
Description:  Tests the ConfigServer

 Copyright (C) 1997-1998 The Hummer Project - hummer@csds.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.test.server;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uidaho.hummer.dataobject.Config;
import edu.uidaho.hummer.test.Test;
import edu.uidaho.hummer.util.StaticMethods;

/**
This class tests that the information is put into and received from the database
through the Configuration Server
 */
public class ConfigServerTest extends Test { 

  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _config_url = new String();
  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _peer_url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _sql_driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _sql_user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  boolean _made_connection = false;
  private String _sql_pwd = new String(); 
  private Socket _client_socket;
  private Properties _constants = new Properties(System.getProperties());
  private int _port;
  Config _config = new Config();  
  InetAddress _sub_addr = null;
  //palouse = 129.101.55.49 this is used to check the peers subscribtion
  //  String _peer_server = new String("129.101.55.49");
  //phobos = 129.101.54.54 this is used to check the peers subscribtion
  //  String _peer_server = new String("129.101.54.54");
  //europa = 129.101.54.53 this is used to check the peers subscribtion
  String _peer_server = new String("129.101.54.53");

  boolean _both = false;

  public ConfigServerTest(String server_name, boolean both) {
    _both = both;
    try {
      _constants = StaticMethods.getHummerProperties();
      Integer value = new Integer(_constants.getProperty("port.CONFIG_PORT"));
      _port = value.intValue();

     }
    catch (FileNotFoundException fnfe) {
      System.err.println("ConfigServerTest: Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("ConfigServerTest: File Hummer.properties not valid.");
      System.exit(0);
    }
    catch (NumberFormatException ne) {
      System.err.println("ConfigServerTest: Bad port.CONFIG_PORT property.");
      System.exit(0);
    }

    try {
      String address = server_name;
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getByName(server_name).getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
	_config_url = _constants.getProperty(
		   "jdbc.url." + address) + nametoken.nextToken();
      }
      else {
	_config_url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }
      
      if (_constants.containsKey("jdbc.driver." + address)) {
	_sql_driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
	_sql_driver = _constants.getProperty("jdbc.driver");
      }
      
      if (_constants.containsKey("jdbc.user." + address)) {
	_sql_user = _constants.getProperty("jdbc.user." + address);
      }
      else {
	_sql_user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
	_sql_pwd = _constants.getProperty("jdbc.password." + address);
      }
      else {
	_sql_pwd = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("ConfigServerTest" + uhe);
    }

    try {
      _sub_addr = InetAddress.getByName(server_name);
    }
    catch(UnknownHostException e){
      System.out.println("ConfigServerTest " + e);
    }

  }

  protected void setTestClassName() {
    _test_class_name = "ConfigServer";
  }

  protected void testDefinition() throws Exception {
    
    prepareSQL();
    insertSQL();
    openServer(_sub_addr);
    if (_made_connection) {
      closeServer();
      checkSQL();
    }
    else {
      System.out.println("There needs to be a configuration server running on " + _sub_addr.getHostName()
                                    + " in order to do a SQL tests on the configuration server");
    }
    
    if (_both) {
      preparePeer();
      insertPeer();    
      openServer(_sub_addr);
      if (_made_connection) {
        closeServer();
        checkPeer();
      }
      else {
        System.out.println("There needs to be a configuration server running on "
                                      + _sub_addr.getHostName()
                                      + "and the host that is in charge of the peer group, " +
                                    " in order to do a Peer tests on the configuration server");
      }
    }
  }

  public boolean openServer(InetAddress sub_name) {
    // Create new socket 
    try  {
      _client_socket = new Socket(sub_name, _port);
      System.out.println("Connected to chat server.");
      _made_connection = true;
    } 
    catch(UnknownHostException e) {
      System.err.println("ConfigServerTest:openServer:Host not found:  " + e.getMessage());
      _made_connection = false;

    }
    catch(IOException e) {
      System.err.println("ConfigServerTest:openServer: " + e.getMessage());
      _made_connection = false;
    }
    return true;
  } //openServer

  /**
  This method sends the data to the server and closes the _client_socket
  to the subordinate.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  public boolean closeServer() {
    ObjectOutputStream out;
    ObjectInputStream in;

    // End of protocol mesage, and close out connection
    try  {
      out = new ObjectOutputStream(_client_socket.getOutputStream());
      out.writeObject(_config);
      in = new ObjectInputStream(_client_socket.getInputStream());
      _config = (Config)in.readObject();
      out.close();
      _client_socket.close();
    }
    catch(UnknownHostException e) {
        System.err.println("ConfigServerTest:closeServer:Host not found:  " + e.getMessage());
        System.exit(1);
    }
    catch(IOException e) {
      System.err.println("ConfigServerTest:closeServer: " + e.getMessage());
      System.exit(1);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("ConfigServerTest:closeServer: " + cnfe);
      System.exit(1);
    }
    return true;
  } //closeServer

  public void prepareSQL() {
    try {
      Class.forName(_sql_driver);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("ConfigServerTest:prepareSQL:" + cnfe);
      System.exit(0);
    }

    try {
      Connection db_connection = DriverManager.getConnection(_config_url, _sql_user, _sql_pwd);
      Statement statement = db_connection.createStatement();
      
      statement.executeUpdate("delete from accept_list;");
      statement.executeUpdate("insert into accept_list values('SelectTest', '129.101.55.96', NULL, 0);");

      statement.executeUpdate("delete from config_time;");
      statement.executeUpdate("insert into config_time values(300);");

      statement.executeUpdate("delete from filter;");
      statement.executeUpdate("insert into filter values('SelectTest', NULL, NULL, NULL, NULL, 1, 1, 1, 1, 1, 0);");

      statement.executeUpdate("delete from forward_list;");
      statement.executeUpdate("insert into forward_list values('SelectTest', '129.101.55.96', 0);");

      statement.executeUpdate("delete from forward_peer;");
      statement.executeUpdate("insert into forward_peer values('SelectTest', 'SelectGroup', 0);");

      statement.executeUpdate("delete from kill_file;");
      statement.executeUpdate("insert into kill_file values('129.101.55.96', NULL, 0);");

      statement.executeUpdate("delete from levels;");
      statement.executeUpdate("insert into levels values('129.101.55.96', NULL, 5, 5, 5, 0);");

      statement.executeUpdate("delete from manager;");
      statement.executeUpdate("insert into manager values('129.101.55.96');");

      statement.executeUpdate("delete from peers;");
      statement.executeUpdate("insert into peers values('SelectGroup', '129.101.55.96', 1, 1);");

      statement.executeUpdate("delete from pgroups;");
      statement.executeUpdate("insert into pgroups values('SelectGroup', '129.101.55.96', 'joel');");

      statement.executeUpdate("delete from subordinates;");
      statement.executeUpdate("insert into subordinates values('129.101.55.96', '129.101.54.54');");

      statement.close();
      db_connection.close();
    }
    catch (SQLException se) {
      System.err.println("ConfigServerTest:prepareSQL:" + se);
    }
  }
  public void insertSQL() {
    String query = new String();

//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests accept_list, 7 select statements
//"SELECT * FROM accept_list;"
    query = "accept_list select";
    _config.data.addElement(query);


//"DELETE FROM accept_list;"
    query = "accept_list delete";
    _config.data.addElement(query);
//"SELECT * FROM accept_list;"
    query = "accept_list select";
    _config.data.addElement(query);

//"INSERT INTO accept_list VALUES ( '" + filter + "' , '" + start_addr + "' , '" 
//                                                       + end_addr + "'," + inh +" );"
    query = "accept_list insert 'configtest1' '129.101.54.54' NULL 0";
    _config.data.addElement(query);
    query = "accept_list insert 'configtest2' '129.101.54.54' NULL 0";
    _config.data.addElement(query);
    query = "accept_list insert 'configtest3' '129.101.54.54' NULL 1";
    _config.data.addElement(query);
//"SELECT * FROM accept_list;"
    query = "accept_list select";
    _config.data.addElement(query);

//"SELECT * FROM accept_list WHERE filter= '" + filter + "' AND inh=" + Inherited +";"
    query = "accept_list select 'configtest1' 0";
    _config.data.addElement(query);

//"SELECT * FROM accept_list WHERE inh=" + Inherited +";"
    query = "accept_list select 1";
    _config.data.addElement(query);

//"DELETE FROM accept_list WHERE filter= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "accept_list delete 'configtest1' 0";
    _config.data.addElement(query);
//"SELECT * FROM accept_list;"
    query = "accept_list select";
    _config.data.addElement(query);

//"INSERT INTO accept_list VALUES ( '" + filter + "' , '" + start_addr + "' , '" 
//                                                       + end_addr + "'," + inh +" );"
    query = "accept_list insert 'configtest1' '129.101.54.54' NULL 0";
    _config.data.addElement(query);
//"DELETE FROM accept_list WHERE inh=" + Inherited +";"
    query = "accept_list delete 0";
    _config.data.addElement(query);
//"SELECT * FROM accept_list;"
    query = "accept_list select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM accept_list;"
    query = "accept_list delete";
    _config.data.addElement(query);


    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests config_time, 3 select statements
//"SELECT * FROM config_time;"
    query = "config_time select";
    _config.data.addElement(query);
    
//"DELETE FROM config_time;"
    query = "config_time delete";
    _config.data.addElement(query);
//"SELECT * FROM config_time;"
    query = "config_time select";
    _config.data.addElement(query);

//"INSERT INTO config_time VALUES(" + NewLevel +");"
    query = "config_time insert 300";
    _config.data.addElement(query);
//"SELECT * FROM config_time;"
    query = "config_time select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM config_time;"
    query = "config_time delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests filter, 7 select statements
//"SELECT * FROM filter;
    query = "filter select";
    _config.data.addElement(query);
    
//"DELETE FROM filter;"
    query = "filter delete";
    _config.data.addElement(query);
//"SELECT * FROM filter;
    query = "filter select";
    _config.data.addElement(query);

//"INSERT INTO filter VALUES ( '" + Expression + "' , " + BeginDate + " , "
//                                               + EndDate + " , " + BeginTime + " , " 
//                                               + EndTime + " , " + Trust + " , " 
//                                               + Coop + " , "  + Integrity + " , " 
//                                               + Log + " , " + Alert + " , " 
//                                               + Inherited +" );"
    query = "filter insert 'testa' NULL NULL NULL NULL 1 1 1 1 0 0";
    _config.data.addElement(query);
    query = "filter insert 'testb' NULL NULL NULL NULL 2 2 2 1 1 0";
    _config.data.addElement(query);
    query = "filter insert 'testb' NULL NULL NULL NULL 2 2 3 1 1 1";
    _config.data.addElement(query);
    query = "filter insert 'testc' NULL NULL NULL NULL 3 3 3 0 1 1";
    _config.data.addElement(query);
//"SELECT * FROM filter;
    query = "filter select";
    _config.data.addElement(query);

//"SELECT * FROM filter WHERE expression= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "filter select 'testb' 0";
    _config.data.addElement(query);

//"SELECT * FROM filter WHERE inh=" + Inherited +";"
    query = "filter select 1";
    _config.data.addElement(query);

//"DELETE FROM filter WHERE expression= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "filter delete 'testb' 1";
    _config.data.addElement(query);
//"SELECT * FROM filter;
    query = "filter select";
    _config.data.addElement(query);

//"DELETE FROM filter WHERE inh=" + Inherited +";"
    query = "filter delete 0";
    _config.data.addElement(query);
//"SELECT * FROM filter;
    query = "filter select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM filter;"
    query = "filter delete";
    _config.data.addElement(query);



    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests forward_list, 6 select statements
//"SELECT * FROM forward_list;
    query = "forward_list select";
    _config.data.addElement(query);
    
//"DELETE FROM forward_list;"
    query = "forward_list delete";
    _config.data.addElement(query);
//"SELECT * FROM forward_list;
    query = "forward_list select";
    _config.data.addElement(query);

//"INSERT INTO forward_list VALUES ( '" + filter + "' , '" + host + "' , " 
//                                                          + Inherited +" );"
    query = "forward_list insert 'testa' '129.101.54.54' 0";
    _config.data.addElement(query);
    query = "forward_list insert 'testb' '129.101.54.54' 0";
    _config.data.addElement(query);
    query = "forward_list insert 'testb' '129.101.54.54' 1";
    _config.data.addElement(query);
    query = "forward_list insert 'testc' '129.101.54.54' 1";
    _config.data.addElement(query);
//"SELECT * FROM forward_list;
    query = "forward_list select";
    _config.data.addElement(query);

//"SELECT * FROM forward_list WHERE filter= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "forward_list select 'testb' 0";
    _config.data.addElement(query);

//"DELETE FROM forward_list WHERE filter= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "forward_list delete 'testb' 0";
    _config.data.addElement(query);
//"SELECT * FROM forward_list;
    query = "forward_list select";
    _config.data.addElement(query);

//"DELETE FROM forward_list WHERE inh=" + Inherited +";"
    query = "forward_list delete 1";
    _config.data.addElement(query);
//"SELECT * FROM forward_list;
    query = "forward_list select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM forward_list;"
    query = "forward_list delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests forward_peer, 5 select statements
//"SELECT * FROM forward_peer;
    query = "forward_peer select";
    _config.data.addElement(query);
    
//"DELETE FROM forward_peer;"
    query = "forward_peer delete";
    _config.data.addElement(query);
//"SELECT * FROM forward_peer;
    query = "forward_peer select";
    _config.data.addElement(query);

//"INSERT INTO forward_peer VALUES ( '" + filter + "' , '" + peergroup + "' , " 
//                                                            + inherited +" );"
    query = "forward_peer insert 'testa' 'testgroup' 0";
    _config.data.addElement(query);
    query = "forward_peer insert 'testb' 'testgroup' 0";
    _config.data.addElement(query);
    query = "forward_peer insert 'testb' 'testgroup' 1";
    _config.data.addElement(query);
    query = "forward_peer insert 'testc' 'testgroup' 1";
    _config.data.addElement(query);
//"SELECT * FROM forward_peer;
    query = "forward_peer select";
    _config.data.addElement(query);

//"SELECT * FROM forward_peer WHERE filter= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "forward_peer select 'testb' 0";
    _config.data.addElement(query);

//"DELETE FROM forward_peer WHERE filter= '" + FilterName + "' AND inh=" + Inherited +";"
    query = "forward_peer delete 'testb' 1";
    _config.data.addElement(query);
//"SELECT * FROM forward_peer;
    query = "forward_peer select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM forward_peer;"
    query = "forward_peer delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests kill_file, 5 select statements
//"SELECT * FROM kill_file;
    query = "kill_file select";
    _config.data.addElement(query);
    
//"DELETE FROM kill_file;"
    query = "kill_file delete";
    _config.data.addElement(query);
//"SELECT * FROM kill_file;
    query = "kill_file select";
    _config.data.addElement(query);

//"INSERT INTO kill_file VALUES ('" + NewKill + "','" + EndAddr + "'," + Inherited + ");"
    query = "kill_file insert '129.101.54.54' NULL 0";
    _config.data.addElement(query);
    query = "kill_file insert '129.101.54.54' NULL 1";
    _config.data.addElement(query);
//"SELECT * FROM kill_file;
    query = "kill_file select";
    _config.data.addElement(query);

//"SELECT * FROM kill_file WHERE inh="+ Inherited +";"
    query = "kill_file select 1";
    _config.data.addElement(query);

//"DELETE  FROM kill_file WHERE start_addr = '" + SelectedKill + "' AND end_addr = '" 
//                                           + EndAddr + "' AND inh = " + Inherited + ";"
    query = "kill_file delete '129.101.54.54' NULL 0";
    _config.data.addElement(query);
//"SELECT * FROM kill_file;
    query = "kill_file select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM kill_file;"
    query = "kill_file delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests levels, 7 select statements
//"SELECT * FROM levels;
    query = "levels select";
    _config.data.addElement(query);
    
//"DELETE FROM levels;"
    query = "levels delete";
    _config.data.addElement(query);
//"SELECT * FROM levels;
    query = "levels select";
    _config.data.addElement(query);

//"INSERT INTO levels VALUES ('" + Startaddr + "','" + EndAddr + "'," + Trust + "," 
//                                               + Coop + ", " + Integrity + ", " + Inherited + ");"
    query = "levels insert '129.101.54.53' NULL 2 2 2 0";
    _config.data.addElement(query);
    query = "levels insert '129.101.54.54' NULL 3 2 1 0";
    _config.data.addElement(query);
    query = "levels insert '129.101.54.54' NULL 1 2 3 1";
    _config.data.addElement(query);
    query = "levels insert '129.101.50.54' NULL 4 4 4 1";
    _config.data.addElement(query);
//"SELECT * FROM levels;
    query = "levels select";
    _config.data.addElement(query);

//"SELECT * FROM levels WHERE inh="+ Inherited +";"
    query = "levels select 1";
    _config.data.addElement(query);

//"SELECT * FROM levels WHERE start_addr='" + Startaddr + "' AND inh= " + Inherited +";"
    query = "levels select '129.101.54.54' 0";
    _config.data.addElement(query);

//"DELETE  FROM levels WHERE start_addr = '" + Startaddr + "' AND end_addr = '" 
//                                           + EndAddr + "' AND inh = " + Inherited + ";"
    query = "levels delete '129.101.54.54' NULL 1";
    _config.data.addElement(query);
//"SELECT * FROM levels;
    query = "levels select";
    _config.data.addElement(query);

//"DELETE FROM levels WHERE inh="+ Inherited +";"
    query = "levels delete 0";
    _config.data.addElement(query);
//"SELECT * FROM levels;
    query = "levels select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM levels;"
    query = "levels delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests manager, 3 select statements
//"SELECT * FROM manager;"
    query = "manager select";
    _config.data.addElement(query);
    
//"DELETE FROM manager;"
    query = "manager delete";
    _config.data.addElement(query);
//"SELECT * FROM manager;"
    query = "manager select";
    _config.data.addElement(query);

//"INSERT INTO manager VALUES ('" + EnterManager + "');"
    query = "manager insert '129.101.55.33'";
    _config.data.addElement(query);
//"SELECT * FROM manager;"
    query = "manager select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM manager;"
    query = "manager delete";
    _config.data.addElement(query);

    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests peers, 9 select statements
//"SELECT * FROM peers;
    query = "peers select";
    _config.data.addElement(query);
    
//"DELETE FROM peers;"
    query = "peers delete";
    _config.data.addElement(query);
//"SELECT * FROM peers;
    query = "peers select";
    _config.data.addElement(query);

//"INSERT INTO peers VALUES ('" + SelectedMod + "','" + NewMember + "',0,0);"
    query = "peers insert 'testa' '129.101.50.50' 1 1";
    _config.data.addElement(query);
    query = "peers insert 'testa' '129.101.54.54' 1 0";
    _config.data.addElement(query);
    query = "peers insert 'testa' '129.101.45.54' 0 0";
    _config.data.addElement(query);
    query = "peers insert 'testb' '129.101.54.54' 1 1";
    _config.data.addElement(query);
    query = "peers insert 'testb' '129.101.45.54' 0 0";
    _config.data.addElement(query);
//"SELECT * FROM peers;
    query = "peers select";
    _config.data.addElement(query);

//"SELECT * FROM peers WHERE pgroup= '" + MainInterface.FullAddress + "';"
    query = "peers select pgroup 'testa'";
    _config.data.addElement(query);

//"SELECT * FROM peers WHERE mem_addr= '" + MainInterface.FullAddress + "';"
    query = "peers select mem_addr '129.101.54.54'";
    _config.data.addElement(query);

//"SELECT * FROM peers WHERE  pgroup= '" + Peer + "' 
//               AND mem_addr= '" + MainInterface.FullAddress + "';"
    query = "peers select 'testa' '129.101.54.54'";
    _config.data.addElement(query);

//"DELETE FROM peers WHERE mem_addr='" + SelectedRemove +"';"
    query = "peers delete mem_addr '129.101.54.54'";
    _config.data.addElement(query);
//"SELECT * FROM peers;
    query = "peers select";
    _config.data.addElement(query);

//"DELETE FROM peers WHERE pgroup = '" + SelectedPeer + "';"
    query = "peers delete pgroup 'testa'";
    _config.data.addElement(query);
//"SELECT * FROM peers;
    query = "peers select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM peers;
    query = "peers delete";
    _config.data.addElement(query);


    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests pgroups, 5 select statements
//"SELECT * FROM pgroups;"
    query = "pgroups select";
    _config.data.addElement(query);
    
//"DELETE FROM pgroups;"
    query = "pgroups delete";
    _config.data.addElement(query);
//"SELECT * FROM pgroups;"
    query = "pgroups select";
    _config.data.addElement(query);

//"INSERT INTO pgroups VALUES ('" + pgroup + "','" + modAddr + "','" 
//                                                   + Email + "');"
    query = "pgroups insert 'testa' '129.101.54.54' 'joel@csds.uidaho.edu'";
    _config.data.addElement(query);
    query = "pgroups insert 'testb' '129.101.54.54' 'joel@csds.uidaho.edu'";
    _config.data.addElement(query);
//"SELECT * FROM pgroups;"
    query = "pgroups select";
    _config.data.addElement(query);

//"DELETE FROM pgroups WHERE pgroup = '" + SelectedPeer + "';"
    query = "pgroups delete 'testa'";
    _config.data.addElement(query);
//"SELECT * FROM pgroups;"
    query = "pgroups select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM pgroups;
    query = "pgroups delete";
    _config.data.addElement(query);


    
//++++++++++++++++++++++++++++++++++++++++++++
//This set of tests, tests subordinates, 4 select statements
//"SELECT * FROM subordinates;"
    query = "subordinates select";
    _config.data.addElement(query);
    
//"DELETE FROM subordinates;"
    query = "subordinates delete";
    _config.data.addElement(query);
//"SELECT * FROM subordinates;"
    query = "subordinates select";
    _config.data.addElement(query);

//"INSERT INTO subordinates VALUES ('" + MainInterface.HumName + "','" 
//                                                          + EnterSubordinate  + "');"
    query = "subordinates insert '129.101.54.53' '129.101.54.54'";
    _config.data.addElement(query);
    query = "subordinates insert '129.101.54.53' '129.101.50.52'";
    _config.data.addElement(query);
//"SELECT * FROM subordinates;"
    query = "subordinates select";
    _config.data.addElement(query);

//"DELETE FROM subordinates WHERE subord = '" + SelectedSub + "';"
    query = "subordinates delete '129.101.54.54'";
    _config.data.addElement(query);
//"SELECT * FROM subordinates;"
    query = "subordinates select";
    _config.data.addElement(query);

//Making sure database is clear
//"DELETE FROM subordinates;
    query = "subordinates delete";
    _config.data.addElement(query);

  }


  public void checkSQL() {
    Enumeration enum = _config.data.elements();
    Enumeration table_enum;
    Enumeration row_enum;
    Vector table = new Vector();
    Vector row = new Vector();
    boolean result = false;

    // These variables are the names of all of the attributes of the database
    // this was done for easier understanding of what was suppose to be received
    String filter = new String();
    String start_addr = new String();
    String end_addr = new String();
    int inh = 0;
    int ctime = 0;
    String expression = new String();
    String date_start = new String();
    String date_end = new String();
    String time_start = new String();
    String time_end = new String();
    int trust = 0;
    int coop = 0;
    int integ = 0;
    int log = 0;
    int alert = 0;
    String host = new String();
    String pgroup = new String();
    String manager = new String();
    String mem_addr = new String();
    int peer_sub = 0;
    int peer_mod = 0;
    String mod_addr = new String();
    String email = new String();
    String hum_name = new String();
    String subord = new String();

    //61
    //7*******************basic acceptList**************************
    newMethod("basic acceptListQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("SelectTest")) && (start_addr.equals("129.101.55.96")) 
         && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete * works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest1")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }

      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest2")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }

      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest3")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest1")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM accept_list WHERE filter= 'filter' AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest3")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM accept_list WHERE inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() ==2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest2")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest3")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM accept_list WHERE filter= 'filter' AND inh=inh; ", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("configtest3")) && (start_addr.equals("129.101.54.54")) 
         && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM accept_list WHERE inh=inh;", result);
    result = false;

    //3*******************basic configList**************************
    newMethod("basic configListQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      ctime = ((Integer)row_enum.nextElement()).intValue();

      if (ctime == 300) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result); 
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
        result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete * works", result); 
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      ctime = ((Integer)row_enum.nextElement()).intValue();

      if (ctime == 300) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result); 
    result = false;

    //7*******************basic filter**************************
    newMethod("basic filterQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("SelectTest")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 1) && (coop == 1) && 
          (integ == 1) && (log == 1) && (alert == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 4) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testa")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 1) && (coop == 1) && 
          (integ == 1) && (log == 1) && (alert == 0) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testb")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 2) && (coop == 2) && 
          (integ == 2) && (log == 1) && (alert == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testb")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 2) && (coop == 2) && 
          (integ == 3) && (log == 1) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testc")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 3) && (coop == 3) && 
          (integ == 3) && (log == 0) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testb")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 2) && (coop == 2) && 
          (integ == 2) && (log == 1) && (alert == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM filter WHERE expression= 'expression' AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testb")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 2) && (coop == 2) && 
          (integ == 3) && (log == 1) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testc")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 3) && (coop == 3) && 
          (integ == 3) && (log == 0) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM filter WHERE inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testa")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 1) && (coop == 1) && 
          (integ == 1) && (log == 1) && (alert == 0) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testb")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 2) && (coop == 2) && 
          (integ == 2) && (log == 1) && (alert == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testc")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 3) && (coop == 3) && 
          (integ == 3) && (log == 0) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM filter WHERE expression= 'expression' AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      expression = (String)row_enum.nextElement();
      date_start = (String)row_enum.nextElement();
      date_end = (String)row_enum.nextElement();
      time_start = (String)row_enum.nextElement();
      time_end = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      log = ((Integer)row_enum.nextElement()).intValue();
      alert = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((expression.equals("testc")) && (date_start == null) && (date_end == null) && 
          (time_start == null) && (time_end == null) && (trust == 3) && (coop == 3) && 
          (integ == 3) && (log == 0) && (alert == 1) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM filter WHERE inh=inh;", result);
    result = false;

    //6*******************basic forwardList**************************
    newMethod("basic forwardListQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("SelectTest")) && (host.equals("129.101.55.96")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 4) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testa")) && (host.equals("129.101.54.54")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (host.equals("129.101.54.54")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (host.equals("129.101.54.54")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testc")) && (host.equals("129.101.54.54")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (host.equals("129.101.54.54")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM forward_list WHERE filter= 'filter' AND inh=inh;", result);
    result = false;


    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testa")) && (host.equals("129.101.54.54")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (host.equals("129.101.54.54")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testc")) && (host.equals("129.101.54.54")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM forward_list WHERE filter= 'filter' AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      host = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testa")) && (host.equals("129.101.54.54")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM forward_list WHERE inh=inh;", result);
    result = false;

    //5*******************basic forwardPeer**************************
    newMethod("basic forwardPeerQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("SelectTest")) && (pgroup.equals("SelectGroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 4) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testa")) && (pgroup.equals("testgroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (pgroup.equals("testgroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (pgroup.equals("testgroup")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testc")) && (pgroup.equals("testgroup")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (pgroup.equals("testgroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if SELECT * FROM forward_peer WHERE filter= 'filter' AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testa")) && (pgroup.equals("testgroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testb")) && (pgroup.equals("testgroup")) 
         && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      filter = (String)row_enum.nextElement();
      pgroup = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((filter.equals("testc")) && (pgroup.equals("testgroup")) 
         && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if DELETE FROM forward_peer WHERE filter= 'filter' AND inh=inh;", result);
    result = false;



    //5*******************basic killFile**************************
    newMethod("basic killFileQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((start_addr.equals("129.101.55.96")) && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete * works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }

    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM kill_file WHERE inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      inh = ((Integer)row_enum.nextElement()).intValue();

      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE  FROM kill_file WHERE start_addr = 'start_addr'"
                            + "AND end_addr = 'end_addr' AND inh = inh;", result);
    result = false;

    //7*******************basic levels**************************
    newMethod("basic levelsQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.55.96")) && (end_addr == null) && (trust == 5) 
           && (coop == 5) && (integ == 5) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 4) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.53")) && (end_addr == null) && (trust == 2) 
           && (coop == 2) && (integ == 2) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (trust == 3) 
           && (coop == 2) && (integ == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (trust == 1) 
           && (coop == 2) && (integ == 3) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.50.54")) && (end_addr == null) && (trust == 4) 
           && (coop == 4) && (integ == 4) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (trust == 1) 
           && (coop == 2) && (integ == 3) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.50.54")) && (end_addr == null) && (trust == 4) 
           && (coop == 4) && (integ == 4) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM levels WHERE inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (trust == 3) 
           && (coop == 2) && (integ == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM levels WHERE start_addr='start_addr'" +
                          " AND inh=inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.53")) && (end_addr == null) && (trust == 2) 
           && (coop == 2) && (integ == 2) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.54.54")) && (end_addr == null) && (trust == 3) 
           && (coop == 2) && (integ == 1) && (inh == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.50.54")) && (end_addr == null) && (trust == 4) 
           && (coop == 4) && (integ == 4) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE  FROM levels WHERE start_addr = 'start_addr'" +
                          "AND end_addr = 'end_addr' AND inh =inh;", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      start_addr = (String)row_enum.nextElement();
      end_addr = (String)row_enum.nextElement();
      trust = ((Integer)row_enum.nextElement()).intValue();
      coop = ((Integer)row_enum.nextElement()).intValue();
      integ = ((Integer)row_enum.nextElement()).intValue();
      inh = ((Integer)row_enum.nextElement()).intValue();
      if ((start_addr.equals("129.101.50.54")) && (end_addr == null) && (trust == 4) 
           && (coop == 4) && (integ == 4) && (inh == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE  FROM levels WHERE inh =inh;", result);
    result = false;

    //3*******************basic manager**************************
    newMethod("basic managerQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      manager = (String)row_enum.nextElement();
      if (manager.equals("129.101.55.96")) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      manager = (String)row_enum.nextElement();
      if (manager.equals("129.101.55.33")) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    //9*******************basic peers**************************
    newMethod("basic peersQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("SelectGroup")) && (mem_addr.equals("129.101.55.96")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 5) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.50.50")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testb")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testb")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.50.50")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM peers WHERE pgroup= 'pgroup';", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testb")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM peers WHERE mem_addr= 'mem_addr';", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.54.54")) 
         && (peer_sub == 1) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing SELECT * FROM peers WHERE  pgroup= 'pgroup'" + 
                            "AND mem_addr= 'mem_addr';", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 3) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.50.50")) 
         && (peer_sub == 1) && (peer_mod == 1)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testa")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testb")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE FROM peers WHERE mem_addr='mem_addr';", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mem_addr = (String)row_enum.nextElement();
      peer_sub = ((Integer)row_enum.nextElement()).intValue();
      peer_mod = ((Integer)row_enum.nextElement()).intValue();

      if ((pgroup.equals("testb")) && (mem_addr.equals("129.101.45.54")) 
         && (peer_sub == 0) && (peer_mod == 0)) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE FROM peers WHERE pgroup = 'pgroup';", result);
    result = false;

    //5*******************basic pgroupsr**************************
    newMethod("basic pgroupsQuery");
    table = (Vector)enum.nextElement();
    if (table.size() ==1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mod_addr = (String)row_enum.nextElement();
      email = (String)row_enum.nextElement();

      if ((pgroup.equals("SelectGroup")) && (mod_addr.equals("129.101.55.96")) 
         && (email.equals("joel"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() ==0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() ==2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mod_addr = (String)row_enum.nextElement();
      email = (String)row_enum.nextElement();

      if ((pgroup.equals("testa")) && (mod_addr.equals("129.101.54.54")) 
         && (email.equals("joel@csds.uidaho.edu"))) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mod_addr = (String)row_enum.nextElement();
      email = (String)row_enum.nextElement();

      if ((pgroup.equals("testb")) && (mod_addr.equals("129.101.54.54")) 
         && (email.equals("joel@csds.uidaho.edu"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() ==1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      pgroup = (String)row_enum.nextElement();
      mod_addr = (String)row_enum.nextElement();
      email = (String)row_enum.nextElement();

      if ((pgroup.equals("testb")) && (mod_addr.equals("129.101.54.54")) 
         && (email.equals("joel@csds.uidaho.edu"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE FROM pgroups WHERE pgroup = 'pgroup';", result);
    result = false;

    //4*******************basic subordinates**************************
    newMethod("basic subordinatesQuery");
    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      hum_name = (String)row_enum.nextElement();
      subord = (String)row_enum.nextElement();
      if ((hum_name.equals("129.101.55.96")) && (subord.equals("129.101.54.54"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if select * works", result);
    result = false;
    
    table = (Vector)enum.nextElement();
    if (table.size() == 0) {
      result = true;
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if delete works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 2) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      hum_name = (String)row_enum.nextElement();
      subord = (String)row_enum.nextElement();
      if ((hum_name.equals("129.101.54.53")) && (subord.equals("129.101.54.54"))) {
        result = true;
      }
      else {
        result = false;
      }
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      hum_name = (String)row_enum.nextElement();
      subord = (String)row_enum.nextElement();
      if ((hum_name.equals("129.101.54.53")) && (subord.equals("129.101.50.52"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if insert works", result);
    result = false;

    table = (Vector)enum.nextElement();
    if (table.size() == 1) {
      table_enum = table.elements();
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      hum_name = (String)row_enum.nextElement();
      subord = (String)row_enum.nextElement();
      if ((hum_name.equals("129.101.54.53")) && (subord.equals("129.101.50.52"))) {
        result = true;
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing DELETE FROM subordinates WHERE subord = 'subord';", result);
    result = false;

    System.out.println("\nDone with ConfigServerTest test\n");
  }


  public void preparePeer() {

    try {
      String address = _peer_server;
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getByName(_peer_server).getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
	_peer_url = _constants.getProperty(
		   "jdbc.url." + address) + nametoken.nextToken();
      }
      else {
	_peer_url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("ConfigServerTest:preparePeer: " + uhe);
    }

    try {
      Class.forName(_sql_driver);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("ConfigServerTest:prepareSQL:" + cnfe);
      System.exit(0);
    }

    try {
      Connection db_connection = DriverManager.getConnection(_peer_url, _sql_user, _sql_pwd);
      Statement statement = db_connection.createStatement();
      
      statement.executeUpdate("delete from peers where pgroup='ConfigServerTest';");
      //europa
      statement.executeUpdate("insert into peers values('ConfigServerTest', '129.101.54.53', 1, 1);");
      //palouse
      statement.executeUpdate("insert into peers values('ConfigServerTest', '129.101.55.49', 1, 0);");
      //phobos
      statement.executeUpdate("insert into peers values('ConfigServerTest', '129.101.54.54', 0, 0);");

      statement.executeUpdate("delete from pgroups where pgroup='ConfigServerTest';");
      statement.executeUpdate("insert into pgroups values('ConfigServerTest', '129.101.55.49', 'joel');");

      statement.close();
      db_connection.close();
    }
    catch (SQLException se) {
      System.err.println("ConfigServerTest:prepareSQL:" + se);
    }
  }

  public void insertPeer() {
    String peer_data = new String();

    _config.data = new Vector();
    peer_data = "subscribe ConfigServerTest " + _peer_server;
    _config.data.addElement(peer_data);
  } 

  public void checkPeer() {
    String query = new String();
    Vector table = new Vector();
    Enumeration enum;
    String pgroup = new String();
    String mem_addr = new String();
    String peer_sub = new String();
    String peer_mod = new String();
    boolean result = false;

    newMethod("usingPeerI");
    try {
      Connection db_connection = DriverManager.getConnection(_peer_url, _sql_user, _sql_pwd);
      Statement statement = db_connection.createStatement();
      query = "select * from peers where pgroup='ConfigServerTest';";
      ResultSet result_set = statement.executeQuery(query);      
      while (result_set.next()){
        pgroup = result_set.getString("pgroup");
        mem_addr = result_set.getString("mem_addr");
        peer_sub = result_set.getString("peer_sub");
        peer_mod = result_set.getString("peer_mod");
        table.addElement(pgroup + " " + mem_addr + " " + peer_sub + " " + peer_mod);
      }
      result_set.close();
      statement.close();
      db_connection.close();
    }
    catch (SQLException se) {
      System.err.println("ConfigServerTest:checkPeer:" + se);
    }
    //    enum = table.elements();
    //    while (enum.hasMoreElements()) {
    //      System.out.println((String)enum.nextElement());
    //    }

    if (table.size() == 3) {
      enum = table.elements();
      if (((String)enum.nextElement()).equals("ConfigServerTest 129.101.54.53 1 1")) {
        if (((String)enum.nextElement()).equals("ConfigServerTest 129.101.55.49 1 0")) {
          if (((String)enum.nextElement()).equals("ConfigServerTest 129.101.54.54 1 0")) {
            result = true;
          }
        }
      }
      else {
        result = false;
      }
    }
    else {
      result = false;
    }
    runSimpleTest("Testing if subscribing to a peer group works", result);
    result = false;

  }//checkPeer

}