/*
Java Hummer Project
File: StaticMethodsTest.java
Date: 2-17-99
Author: Chad Ward
Description:  Tests StaticMethods class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.util;

import java.security.*;
import java.util.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.test.Test;

public class StaticMethodsTest extends Test {

        /**
        Main method so that this test can be run from the command line
        */
  public static void main(String[] argv) throws Exception {
    Test test_object = new StaticMethodsTest();
    test_object.runClassTest(1);
  }


	/**
	Initializes the _test_class_name variable.
	*/
  protected void setTestClassName() {
    _test_class_name = "StaticMethods";
  }

  protected void testDefinition() throws Exception {
    String[] current_value;
    final String[] _equals_test1_1 = {"hello", "there", "test"},
                   _equals_test1_2 = {"hello", "there", "test", "again"},
                   _equals_test1_3 = {"hello", "there", "test1"},
                   _split_ans1 = {"12-08-1998", "10:41:17", "WHONAMES#root&&1&#ward9675&&2&#" +
                                      "york9445&muskrat.cs.uidah/muskrat.cs:0.0/muskrat.cs:0.0/muskrat.cs:0.0&&6&#" +
                                      "postgres&&0&#muel9560&129.101.55:10.0&0&",
                                  "kootenai.cs.uidaho.edu"},
                   _split_ans2_1 = { "york9445", "muskrat.cs.uidah/muskrat.cs:0.0/muskrat.cs:0.0/muskrat.cs:0.0",
                                     "6", ""},
                   _split_ans2_2 = { "york9445", "muskrat.cs.uidah/muskrat.cs:0.0/muskrat.cs:0.0/muskrat.cs:0.0",
                                      "", "6", ""};
    final String _split_test1 = "12-08-1998|10:41:17|WHONAMES#root&&1&#ward9675&&2&#" +
                                "york9445&muskrat.cs.uidah/muskrat.cs:0.0/muskrat.cs:0.0/muskrat.cs:0.0&&6&#" +
                                "postgres&&0&#muel9560&129.101.55:10.0&0&|kootenai.cs.uidaho.edu";
    KeyPair key_pair = StaticMethods.generateKeyPair();

		/* ***********************
		equals testing
		*************************/
    StaticMethods static_obj = new StaticMethods();
    newMethod("equals");
    runSimpleTest("Two non-null equal arrays", "true",
                  StaticMethods.equals(_equals_test1_1, _equals_test1_1));
    runSimpleTest("Two non-null equal size, unequal arrays", "false",
                  !StaticMethods.equals(_equals_test1_1, _equals_test1_3));
    runSimpleTest("Two non-null unequal size arrays", "false",
                  !StaticMethods.equals(_equals_test1_1, _equals_test1_2));
    runSimpleTest("One null and one non-null array", "false",
                  !StaticMethods.equals(_equals_test1_1, (String[])null));
    runSimpleTest("Two null arrays", "true",
                  StaticMethods.equals(null, null));
    runSimpleTest("Two empty arrays", "true",
                  StaticMethods.equals(new String[0], new String[0]));

		/* ***********************
		split testing
		*************************/
    newMethod("split");
    current_value = new String[0];
    runExceptionTest("Did not throw exception for null String", static_obj,
                     "split", null, false, (String)null, ",", Boolean.TRUE);
    runSimpleTest("Null String (repeat=true)", "0 length array",
                  StaticMethods.split(null, ",", true).length == 0);
    runSimpleTest("Null String (repeat=false)", "0 length array",
                  StaticMethods.split(null, ",", false).length == 0);

    current_value = StaticMethods.split(_split_test1, "|", true);
    runSimpleTest("String with singleton seperators (repeat=true)", "Known valid return value",
                  StaticMethods.equals(_split_ans1, current_value));

    current_value = StaticMethods.split(_split_test1, "|", false);
    runSimpleTest("String with singleton seperators (repeat=false)", "Known valid return value",
                  StaticMethods.equals(_split_ans1, current_value));

    String split_test2 = StaticMethods.split(StaticMethods.split(_split_test1, "|")[2], "#")[3];
    current_value = StaticMethods.split(split_test2, "&", true);
    runSimpleTest("String with adjacent seperators (repeat=true)", "Known valid return value",
                  StaticMethods.equals(_split_ans2_1, current_value));

    current_value = StaticMethods.split(split_test2, "&", false);
    runSimpleTest("String with adjacent seperators (repeat=false)", "Known valid return value",
                  StaticMethods.equals(_split_ans2_2, current_value));

		/* ***********************
		getBytes testing
		*************************/
    newMethod("getBytes");
    runSimpleTest("Null object", "5", (StaticMethods.getBytes(null).length == 5));
    runSimpleTest("Known size object (String \"Hello\")", "12",
                 (StaticMethods.getBytes("Hello").length == 12));

		/* ***********************
		getRandomTimestampBytes testing
		*************************/
    newMethod("getRandomTimestampBytes");
    runExceptionTest("Did not throw exception (no parameters).", static_obj,
                     "getRandomTimestampBytes", null, false);

		/* ***********************
		getDigest testing
		*************************/
    newMethod("getDigest");
    runExceptionTest("Non-null values (bob, dog)", static_obj, "getDigest",
                     null, false, "bob".getBytes(), "dog".getBytes(), key_pair.getPublic());

    runExceptionTest("Non-null alias and null password", static_obj, "getDigest",
                     null, false, "bob".getBytes(), null, key_pair.getPublic());

    runExceptionTest("Null alias and non-null password", static_obj, "getDigest",
                     null, false, null, "dog".getBytes(), key_pair.getPublic());

    runExceptionTest("Null alias and null password", static_obj, "getDigest",
                     null, false, null, null, key_pair.getPublic());

    runExceptionTest("Null alias, null password and null PublicKey", static_obj, "getDigest",
                     null, true, null, null, null);

		/* ***********************
		generateKeyPair testing
		*************************/
    newMethod("generateKeyPair");
    runExceptionTest("Takes no input", static_obj, "generateKeyPair", null, false);

		/* ***********************
		getSignatureObject testing
		*************************/
    newMethod("getSignatureObject");
//    runExceptionTest("Takes no input", static_obj, "getSignatureObject", null, false);

		/* **********************
		responseToString testing
		************************/
    newMethod("responseToString");
    Response response_object = new Response();
    response_object._type = 0;
    response_object._id = 0;
    response_object._message = "The message";
    response_object._data = null;
    String indent = "  ",
           header = (indent + "type: 0\n" + indent + "id: 0\n" + indent +
                     "message: The message\n" + indent + "data -\n"),
           data_string = "";
    runSimpleTest("Null input", "Emtpy String", StaticMethods.responseToString(null, indent) == "");
    runSimpleTest("Null data Vector", "Empty values for data information",
                  StaticMethods.responseToString(response_object, indent).equals(header + data_string));
    response_object._data = new Vector();
    runSimpleTest("data Vector contains no values", "Empty values for data information",
                  StaticMethods.responseToString(response_object, indent).equals(header + data_string));

    response_object._data.add("Value1");
    response_object._data.add("Value2");
    response_object._data.add("Value3");
    data_string = indent + indent + "data[0](java.lang.String): Value1\n" +
                  indent + indent + "data[1](java.lang.String): Value2\n" +
                  indent + indent + "data[2](java.lang.String): Value3\n";
    runSimpleTest("data Vector with three String values (\"Value1\", \"Value2\", \"Value3\")",
                  "String describing _type, _id, _message, and _data with class information",
                  StaticMethods.responseToString(response_object, indent).equals(header + data_string));

		/* ***********************
		byteToHexString testing
		*************************/
    newMethod("byteToHexString");
    byte[] hex_test_array1 = {(byte)163}, hex_test_array2 = new byte[0],
           hex_test_array3 = {(byte)255, (byte)23, (byte)1};
    String array_ans1 = "A3", array_ans2 = "", array_ans3 = "FF 17  1";
    runSimpleTest("Single value array (163)", "\"" + array_ans1 + "\"",
                  StaticMethods.byteToHexString(hex_test_array1).equals(array_ans1));
    runSimpleTest("Zero value array (Zero length)", "Empty string",
                  StaticMethods.byteToHexString(hex_test_array2).equals(array_ans2));
     runSimpleTest("Multiple value array (255, 23, 18)", "\"" + array_ans3 + "\"",
                  StaticMethods.byteToHexString(hex_test_array3).equals(array_ans3));
 }
}
