package edu.uidaho.hummer.tool;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uidaho.hummer.test.Test;


public class InitializeTablesTest extends Test{
  /**
     boolean value to determine whether or not a connection to the database was successful.
  */
  public boolean _isConnected = false;
  /**
     BufferedReader stores the data from the file used to initialize the database.
  */
  public BufferedReader _buffer = null;
  /**
     String representing the machine that is accessing the database.
  */
  public String sql_user = new String();
  /** 
      String representing the url where the database is located.
  */
  public String sql_url= new String();
  /**
     String that holds the password to the database.
  */
  public String sql_password = new String(); 
  public Connection connection = null;
  public Statement statement = null;
  public Vector test = new Vector(); 
  public Vector _drop_tables = new Vector();
 
 

  public InitializeTablesTest(){
    Properties _constants = new Properties(System.getProperties());
   
    FileInputStream _file = null;
    String sql_driver = new String(); 
    
    try {
     
      String user_dir = _constants.getProperty("user.dir");
      String file_separator = _constants.getProperty("file.separator");
      _file =  new FileInputStream(user_dir + file_separator +
				   "Hummer.properties");
     
      _constants.load(new BufferedInputStream(_file));
       _file.close();
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("InitializeTables: Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("InitializeTables: File Hummer.properties not valid.");
      System.exit(0);
    }
    catch (NumberFormatException ne) {
      System.err.println("InitializeTables: Bad port.MESSAGE_PORT property.");
      System.exit(0);
    }  
    
    
    try {
      String address = InetAddress.getLocalHost().getHostAddress();
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {

	sql_url = _constants.getProperty("jdbc.url." + address)
	  + nametoken.nextToken();
      }
      else {

	sql_url = _constants.getProperty("jdbc.url") 
	  + nametoken.nextToken();

      }      
      if (_constants.containsKey("jdbc.driver." + address)) {
	sql_driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
	sql_driver = _constants.getProperty("jdbc.driver");
      }

      if (_constants.containsKey("jdbc.user." + address)) {
	sql_user = _constants.getProperty("jdbc.user." + address);
      }
      else {
	sql_user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
	sql_password = _constants.getProperty("jdbc.password." + address);
      }
      else {
	sql_password = _constants.getProperty("jdbc.password");
      }		
     
      Class.forName(sql_driver);       
      _isConnected = true;
    }
    catch (UnknownHostException uhe) {
      System.err.println("InitializeTables:" + uhe);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("InitializeTables:" + cnfe);
      System.exit(0);
    }      
  }

  protected void testDefinition()throws Exception{
  
    sqlStatements();
      if(_isConnected){ 
	dropTables();
	createTables();
	openFile();
	initializeTables();
	try{
	  connection.close();
	}
	catch(SQLException e){
	  System.err.println("InitializeTablesTest: " +e);
	}
      }
      else{
	System.err.println("There was an error connecting to the database");
      }

  }
	
  protected void setTestClassName(){
    _test_class_name = "InitializeTables";
  }
  
  public void sqlStatements() throws SQLException {
    if (_isConnected) {
      try{
	connection = DriverManager.getConnection
	  (sql_url, sql_user, sql_password); 
	statement= connection.createStatement();
      }
      catch(SQLException e){
	System.err.println("InitializeTables: sqlStatements: " +e);
      }
    }
  }
  
  public void dropTables(){
    /*The database should drop all tables successfully if the tables already exist.  
      To check the tables the sql query in the form " Select * from (tablename)" 
      will be used. If this is successful we should recieve NO errors when dropping
      the table.  Otherwise an error will occur.*/
    
    String select = new String();
    String drop   = new String();
    boolean result = true;
    
    try{
    select = "select * from log;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table log;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("log table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("log table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }


    /***************************************************************/
    try{
    select = "select * from filter;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table filter;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("filter table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("filter table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }

 /***************************************************************/
    try{
    select = "select * from accept_list;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table accept_list;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("accept_list table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("accept_list table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }




 /***************************************************************/
    try{
    select = "select * from forward_list;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table forward_list;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("forward_list table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("forward_list table exists and we dropped table.",
		    "DROPPED", result);
    }

 /***************************************************************/
    try{
    select = "select * from forward_peer;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table forward_peer;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("forward_peer table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("forward_peer table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }

 /***************************************************************/
    try{
    select = "select * from manager;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table manager;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("manager table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("manager table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }

/***************************************************************/
    try{
    select = "select * from levels;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table levels;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("levels table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("levels table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from config_time;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table config_time;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("config_time table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("config_time table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }

/***************************************************************/
    try{
    select = "select * from subordinates;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table subordinates;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("subordinates table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("subordinates table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from pid;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table pid;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("pid table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("pid table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from kill_file;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table kill_file;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("kill_file table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("kill_file table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from pgroups;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table pgroups;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("pgroups table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("pgroups table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from peers;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table peers;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("peers table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("peers table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
/***************************************************************/
    try{
    select = "select * from message;";
    statement.executeUpdate(select);
    statement.close();
    }
    catch(SQLException e){
      result = false;
    }

    try{
    drop = "drop table message;";
    statement.executeUpdate(drop);
    statement.close();
    }
    catch(SQLException e){
    }

    if(result == false){
    runSimpleTest("message table did not exist, and we dropped table",
		  "java.sql.SQLException: ERROR:  Relation log Does Not Exist!",!result);
    }
    else{
      runSimpleTest("message table exists and we dropped table.",
		    "java.sql.SQLException: ERROR:  Relation log Does Not Exist!", result);
    }
}

  public void createTables(){
    String S = new String();
      S = "CREATE TABLE log (" +
	"trust int,"+	
	"coop int," +
	"integ int," +
	"ldate date,"+
	"ltime date," +
	"description text," +
	"hostip text," +
	"file text," +
	"tool text);";
      test.addElement(S);
      
      S = "CREATE TABLE filter (" +
	"expression varchar,"+
	"date_start date,"+
	"date_end date,"+
	"time_start varchar,"+
	"time_end varchar,"+
	"trust int,"+
	"coop int,"+
	"integ int,"+
	"log int,"+
	"alert int,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE accept_list ("+
	"filter text,"+
	"start_addr text,"+
	"end_addr text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE forward_list ("+
	"filter text,"+
	"host text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE forward_peer ("+
	"filter text,"+
	"pgroup text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE manager ("+
	"manager text);";
      test.addElement(S);
      
      S = "CREATE TABLE levels ("+
	"start_addr text,"+
	"end_addr text,"+
	"trust int,"+
	"coop int,"+
	"integ int,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE config_time ("+
	"ctime int);";
      test.addElement(S);
      
      S = "CREATE TABLE subordinates ("+
	"hum_name text,"+
	"subord text);";
      test.addElement(S);
      
      S = "CREATE TABLE pid ("+
	"name text,"+
	"pid int);";
      test.addElement(S);
      
      S = "CREATE TABLE kill_file ("+
	"start_addr text,"+
	"end_addr text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE pgroups ("+
	"pgroup text,"+
	"mod_addr text,"+
	"email text);";
      test.addElement(S);
      
      S = "CREATE TABLE peers ("+
	"pgroup text,"+
	"mem_addr text,"+
	"peer_sub int,"+
	"peer_mod int);";
      test.addElement(S);
      
      S = "CREATE TABLE message ("+
	"pgroup text,"+
	"mem_addr text,"+
	"action text,"+
	"rdate date,"+
	"rtime varchar,"+
	"status int,"+
	"sdate date,"+
	"stime varchar,"+
	"mpublic int);";
      test.addElement(S);

      /************************************************/
      /*Run the test for each table*/
      boolean result = true;
      Enumeration enum = test.elements();
      String table = new String();

      while(enum.hasMoreElements()){
	try{
	  table = new String((String)enum.nextElement());
	  statement.executeUpdate(table);
	  statement.close();
	}
	catch(SQLException e){
	
	result = false;
	  runSimpleTest(table,e.getMessage(), result);
	}
	if(result == true){
	  runSimpleTest(table ,"CREATE",result);
	}
	result = true;
    }
  }

  public void openFile(){
    boolean result = true;
    FileInputStream _file = null;
    Properties _constants = new Properties(System.getProperties());
    String user_dir = _constants.getProperty("user.dir");
    String file_separator = _constants.getProperty("file.separator");
    
    /*test open Hummer file*/
    try{
      _file =  new FileInputStream(user_dir + file_separator +
				   "Hummer.properties");
    }
    catch(FileNotFoundException f){
      result = false;
    }
    if(result == false){
      runSimpleTest("Failed to open Hummer.property file",result);
    }
    else{
    runSimpleTest("Open Hummer.property file",result);
    
    try{
    _constants.load(new BufferedInputStream(_file));
    }
    catch(IOException i){
    }

    /*test close file*/
    try{
      _file.close();
    }
    catch(IOException io){
      result = false;
    }
    if(result == false){
      runSimpleTest("Failed to close Hummer.property file",result);
    }
    else{
      runSimpleTest("close Hummer.property file",result);
    }

    String _logfile = new String();
    FileReader _log = null;

    result = true;

    if(_constants.containsKey("jdbc.tool.add_values")){
      _logfile = new String(user_dir +file_separator 
				   +_constants.getProperty
				   ("jdbc.tool.add_values"));
    }

    /*test add_values file*/
    try{
      _log = new FileReader(_logfile);
      _buffer = new BufferedReader(_log);
    }
    catch(FileNotFoundException f){
      result = false;
    }
    if(result == false){
      runSimpleTest("Failed to open add_values.sql file",result);
    }
    else{
      runSimpleTest("Open add_values.sql",result);
    }
    }
  }


  public void initializeTables(){
  }
}
