/*
Java Hummer Project
File: BdfTool.java
Date: 12-30-98
Author: Tiffany Cavanaugh
Description:   This tool is designed to parse the bdf system tool and send all  relevant information to its hummer.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.tool.SunOS;

import java.io.*;
import java.util.*;

import edu.uidaho.hummer.tool.HummerTool;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.client.MessageI;
import edu.uidaho.hummer.util.HummerKeyStore;


/**
The BdfTool class is a thread in the class ToolThreads. BdfTool sends a message to MessageI every five minutes.

<h4>Class Usage and Flow:</h4>
<ul>
<li> Constructor creates the thread with the jdbc.tools.bdftool property from the constants object.
<li>The start() methed is launched because it is a thread.
<li>Loops the thread on boolean variables _exit_value and _exit_status.
<ul>
<li>I gets the local system informatioin running df and calls MessageI which sendsthe message to the database running.
<li>The thread is terminated when the MessagingServer is terminated.
</ul>

@author Tiffany Cavanaugh, Chad Ward
*/
public class BdfTool extends HummerTool {

	/**
	Creates the thread BdfTool as a part of the thread group created by ToolThreads.
	@param constants A Hummer properties file being passed in which removes
		the extra construction time for Properties objects.
	*/
  public BdfTool(Properties constants, ThreadGroup thread_group, HummerKeyStore key_store){
    super("BDF", thread_group, key_store);
    _logfile = constants.getProperty("jdbc.tools.bdftool");
  }
  
	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Returns a human readable description of this tool.
	</ul>
	*/
  public String getDescription() {
    String description = "Returns the amount of disk space being used by the system.  " +
                         "This value is gathered from a the Unix bdf tool.";
    return description;
  }


	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>This method is where the information from the logfile is gathered and
	  	parsed by implementing a separate process.
	  <li>The data is put into a buffer in order to manipulate it accordingly.  
	  <li>Then it is sent as a message to MessageI.
	</ul>
	@param _logfile The buffer where all the data collected from system call is
		stored.
	*/
  public void gatherInformation() { 
    BufferedReader output = null;
    int exit_status = 0, exit_value = 0;
    StringTokenizer token = null;
    String delimitor = " ", this_line, dev, blocks, used,
           avail, capasity, mnt, parser, msngr = "BDF: ";

    Vector information = new Vector();


    Runtime log = Runtime.getRuntime();
    Process process = null;
    try{
      process = log.exec(_logfile);   
      output  = new BufferedReader(new InputStreamReader(process.getInputStream()));
      exit_status = process.waitFor();    
      process.destroy();
      exit_value = process.exitValue();      
      log.gc();
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("BdfTool:gatherInformation:" + e);
    }
    catch(InterruptedException e){
      _status = "Error: " + e;
      System.err.println("BdfTool:gatherInformation:" + e);
    }     

    try{
      while((this_line = output.readLine()) != null ){
        if(!this_line.startsWith("Filesystem")) {
          token = new StringTokenizer(this_line,delimitor);
          dev = token.nextToken();
          blocks = token.nextToken();
          used = token.nextToken();
          avail =  token.nextToken();
          capasity =  token.nextToken();
          mnt =  token.nextToken();
        
          capasity = capasity.replace('%', ' ');
          information.addElement(msngr.concat(dev +' ' +capasity));
        }
      }
  
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("BdfTool: stringManipulator: " +e);
    }
    
    try{
      output.close();
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("BdfTool: stringManipulator: " +e);
    }
    MessageI outputToMessageI = new MessageI(_key_store);
    Message param_data = new Message();
    Enumeration enum;
    enum = information.elements();
    while(enum.hasMoreElements()) {
      String   doTool[] = {"m ", (String) enum.nextElement(),  "h " ,_host_name,
                           "f " ,_logfile,  "s " ,"bdftool"};

      outputToMessageI.parseParameters(doTool, param_data);
      outputToMessageI.validateParameters(param_data);
      long time = System.currentTimeMillis();
      outputToMessageI.openMessageConnection(_host_name);
      outputToMessageI.sendToHummer(param_data);
      outputToMessageI.closeMessageConnection();
    }
  }
}