/*
Java Hummer Project
File: HummerSocketException.java
Date: 1-25-99
Author: Chad Ward
Description:  Exception thrown when a HummerSocket connection cannot be made correctly.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

import edu.uidaho.hummer.dataobject.Response;

/**
Exception thrown when a HummerSocket connection cannot be made correctly.
@author Chad Ward
*/
public class HummerSocketException extends Exception {

	/**
	Current information that was recieved when the error occured
	@serial
	*/
  public Response _recieved_information = null;

	/**
	Type of exception being thrown.
	@see HummerSocketException#_UNSPECIFIED
	@see HummerSocketException#_PROTOCOL_NOT_KNOWN
	@see HummerSocketException#_INVALID_DATA_FORMAT
	@see HummerSocketException#_UNKNOWN_ALIAS
	@see HummerSocketException#_INVALID_LOGIN
	@see HummerSocketException#_INVALID_SIGNATURE
	@see HummerSocketException#_UNKNOWN_HUMMER_CERTIFICATE_SIGNER
	@see HummerSocketException#_EXPIRED_HUMMER_CERTIFICATE
	@see HummerSocketException#_HUMMER_CERTIFICATE_SIGNER_ACCOUNT_UNITIALIZED
	@see HummerSocketException#_NON_TRUSTED_HUMMER_CERTIFICATE
	@see HummerSocketException#_INVALID_MESSAGE_ID
	@see HummerSocketException#_INVALID_COMMAND
	@serial
	*/
  public int _type = 0;

	/**
	Type of exception being thrown.
	@see HummerSocketException#_type
	@see HummerSocketException#_UNSPECIFIED
	@see HummerSocketException#_PROTOCOL_NOT_KNOWN
	@see HummerSocketException#_INVALID_DATA_FORMAT
	@see HummerSocketException#_UNKNOWN_ALIAS
	@see HummerSocketException#_INVALID_LOGIN
	@see HummerSocketException#_INVALID_SIGNATURE
	@see HummerSocketException#_UNKNOWN_HUMMER_CERTIFICATE_SIGNER
	@see HummerSocketException#_EXPIRED_HUMMER_CERTIFICATE
	@see HummerSocketException#_HUMMER_CERTIFICATE_SIGNER_ACCOUNT_UNITIALIZED
	@see HummerSocketException#_NON_TRUSTED_HUMMER_CERTIFICATE
	@see HummerSocketException#_INVALID_MESSAGE_ID
	@see HummerSocketException#_INVALID_COMMAND
	@serial
	*/
  public final static int _UNSPECIFIED = 0x0,
                          _PROTOCOL_NOT_KNOWN = 0x1,
                          _INVALID_DATA_FORMAT = 0x2,
                          _UNKNOWN_ALIAS = 0x4,
                          _INVALID_LOGIN = 0x8,
                          _INVALID_SIGNATURE = 0x10,
                          _UNKNOWN_HUMMER_CERTIFICATE_SIGNER = 0x20,
                          _EXPIRED_HUMMER_CERTIFICATE = 0x40,
                          _HUMMER_CERTIFICATE_SIGNER_ACCOUNT_UNITIALIZED = 0x80,
                          _NON_TRUSTED_HUMMER_CERTIFICATE = 0x100,
                          _INVALID_MESSAGE_ID = 0x200,
                          _INVALID_COMMAND = 0x400;

	/**
	Calls constructor of Exception with <i>e</i>, and sets _recieved_information.
	@param e Exception description
	@see HummerSocketException#_type
	*/
  public HummerSocketException(String e, Response recieved_information, int type) {
    super(e);
    this._recieved_information = recieved_information;
    this._type = type;
  }
}