/**
DummyServer.java
Joel Marlow
Fall 1998
Hummingbird Project - Dr. Frincke
Coded by: Joel Marlow
Description: basic server

 Copyright (C) 1997-1999 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*****************************************************************/


import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import java.util.StringTokenizer;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.util.socket.*;
import edu.uidaho.hummer.dataobject.*;

/**
The DummyServer class is the main access point for peer subscribing, peer unsubscribing
and peer messaging processing in Hummer. It connects to the socket specified by the 
constructor and waits for messages. When it receives a connection it uses the procRequest
method. Which will figure out what to do with the information received from the socket.   

@author Joel Marlow
*/

public class DummyServer {
  /**
  The server socket for the DummyServer.  This may not need to be a class
  variable.
  */
  //++Needed for authentication
  HummerServerSocket _hummer_socket;
  public ServerSocket _server_socket;
  /**
  int representing the port number the MessagingServer should be running on.
  The value of this int is set in the Hummer.properties file and should only be
  set via the _constants Properties object.
  */
  public int _port = 16000; 
  /**
  Boolean value representing whether or not the DummyServer should listen
  for additional connections.
  */
  public boolean _server_run = false;

  public Socket _client_socket = null;
  //++Needed for authentication
  HummerKeyStore _key_store;

/**
Creates the DummyServer using the port specified in the call.

*/


  public DummyServer() {
  //++This try and catch is needed for authentication
    try {
    String address = InetAddress.getLocalHost().getHostAddress();
    _key_store = new HummerKeyStore();
    _key_store.load(new FileInputStream(address +"_key.hum"));
    }
    catch (Exception e) {
      System.err.println("DummyClient: " + e);
    }

    _server_run = false;
    // Create the sockets and initialize.

    try {
      _server_socket = new ServerSocket (_port);
      _server_run = true;
    } 
    catch (IOException e) {
      System.err.println ("Could not create server");
    }
  }


  /**
  This is the place the DummyServer spends its time until it receives a 
  connection. After initializing the server socket the execution moves into
  a period of looping, waiting for socket connections.  Access to this loop
  is governed by a boolean value that can be set to end the loop by calling
  the method killMessagingServer().
  */
  public void startServer() {
    String peer_addr = new String();  
    
    _server_run = true;
    
    while (_server_run) { 
      try {
        String address = InetAddress.getLocalHost().getHostAddress();
        // wait to accept socket connections
        System.out.println ("DummyServer waiting for clients...");
        _client_socket = _server_socket.accept();
        //++Needed for authentication
        _hummer_socket = new HummerServerSocket(null, _client_socket, _key_store);
        //++Needed for authentication
        _hummer_socket.validation(null); 
        //writing out the keystore so there is no trouble
        _key_store.store(new FileOutputStream(address +"_key.hum"));


        processRequest(_client_socket);
        _client_socket.close();
      } 
      catch (IOException ioe) {
        System.err.println ("DummyServer:startServer:" + ioe);
      }
      //++Needed for authentication this one should look
      //++for all of the exceptions instead of the general one
      catch(Exception e) {
        System.err.println ("DummyServer:startServer:" + e);
      }

    } //while  
    try {
      _server_socket.close();
    }
    catch (IOException ioe) {
      System.err.println("DummyServer:startServer:" + ioe);
    }

  } //startServer



  public boolean processRequest(Socket peer_addr) {
    Vector store_config = new Vector();
    Enumeration enum; 
    String query = new String ();
    Config config = new Config();
    //++Needed for authentication
    Response response;
    boolean more = true;

    try {
      while (more) {
        //++Needed for authentication
	//              System.out.println("has validated:" + _hummer_socket.hasValidated());
        response = _hummer_socket.readResponseObject();
        //++Needed for authentication
        config = (Config)response._message;
	//	System.out.println("first");
        store_config = config.data;
	//            System.out.println("second");
        config.data = new Vector();
        enum = store_config.elements();
        query = (String)enum.nextElement();
        if (query.equals("close")) {
          more = false;
        }
        System.out.println(query);
        query = query + " Yes! I did";
        config.data.addElement(query);
        //++Needed for authentication
        response = new Response();
        //++Needed for authentication
        response._type= Response._OK;
        //++Needed for authentication
        response._message = config;
        //++Needed for authentication
        //      System.out.println("has validated:" + _hummer_socket.hasValidated());
        _hummer_socket.writeResponseObject(response);
      }

      //++Needed for authentication
      _hummer_socket.close();
    }
    catch (IOException I) { 
      //      System.out.println("first");
      System.err.println ("DummyServer:processRequest: " + I); 
      //      I.printStackTrace();
      return false;
    }   
      //++Needed for authentication this one should look
      //++for all of the exceptions instead of the general one
    catch (Exception e) {
      //            System.out.println("third");
      System.err.println("DummyServer:processRequest:" + e);
            e.printStackTrace();
    }
    return true;
  }

  /**
     The executable code.
  */
  public static void main (String args[]) {

    DummyServer ds = new DummyServer();
    try {
      ds.startServer();
    }
    catch (NullPointerException n) {
      System.err.println("DummyServer:" + n);
    }
  }
} 
